package project;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

/**
 *	This class is the frame containing our about.
 *
 *	@author Hans Stenten
 */
public class AboutFrame extends Frame{
	private URL fFileLocation;

        Panel generalPanel = new Panel();
        Panel paddingBottom = new Panel();
        BorderLayout borderLayout1 = new BorderLayout();
        Panel bodyPanel = new Panel();
        BorderLayout borderLayout2 = new BorderLayout();
        Panel fScroller = new Panel();
        GridLayout gridLayout2 = new GridLayout();
        Button aboutButton = new Button();

        /**
         * Constructor of the frame, takes a URL pointing to the directory were
         * help files can be found. And makes the helpframe
         *
         * @param       directory       pointing to the help files directory
         *
         * @author      Filip Van Rysselberghe
         * @author      Hans Stenten
         */
        public AboutFrame( URL directory ) {
                try {
                        fFileLocation = directory;
                        jbInit();
                }
                catch(Exception e) {
                        e.printStackTrace();
                }
        }


	/**
         *	Eventhandler for the frame itself. Eventhandling for the
	 *	buttons is performed wherever they are declared. This
	 *	function only reacts on a window-destroy event by disposing
	 *	itself
         *
         * 	@author Filip Van Rysselberghe
         */
	public boolean handleEvent(Event e ) {
		if ( e.id == Event.WINDOW_DESTROY) {
			dispose();
		}
		return super.handleEvent( e );
	}

	/**
	 *	Shows a certain subject (helpsubject) on the screen. First of all
	 *	it creates a panel and adds the title of the text. Then the
	 *	text is added below the title. Below the text comes an extra
	 *	line which you could see asa seperator.
	 *
	 *	@param	String	the title of the subject
	 *	@param	Text	the text corresponding the subject
	 *
	 *	@author Filip Van Rysselberghe
	 */
	public void showSubject(String title, Text subject){
		int estimatedHeight = ( (subject.length()+1) * 14 ) + 80 + 45 + 20;
		final int realHeight = (estimatedHeight<300)?300:estimatedHeight;

		//maybe the solution for our scrollpane problem
		ImagePanel textField = new ImagePanel();

		textField.setLocation( 0,0 );
		textField.setLayout(null);
		textField.setSize(320, realHeight);

		textField.setBackground(Color.white);
		textField.setVisible(true);

		fScroller.removeAll();
		fScroller.add(textField);

		//drawing of the subject
		//->First we'll create an image using the panel on which
		//we 're going to draw.
		Image img = textField.createImage(textField.getSize().width, textField.getSize().height);
		Graphics g = img.getGraphics();

		//->title
		g.setFont( new Font("sansserif", Font.BOLD, 20) );
		g.setColor( Color.orange );
		g.drawString( title, 40, 40);
                printText(g, subject);

		textField.paint(img);
	}

        /**
         *      This adds the text of the subject below the title.
         *      The help panel has a maximum of 21 lines it can display,
         *      this function makes sure the correct lines of text are displayed.
         *      The upper and lower limit are determined by the scrollbar.
         *
         *      @param  g       graphics of the imagepanel, text is drawn on this one
         *      @param  subject the text this function displays
	 *
	 *	@author Filip Van Rysselberghe
         *      @author Hans Stenten
	 */
        private void printText(Graphics g, Text subject){
                // neem aan 300 hoog, 320 breed
                //->text
		g.setFont( new Font("Helvetica", Font.PLAIN, 12) );
		g.setColor( Color.black);
		int	x = 20,
			y = 80,
			linefeed = 14;

		for (int i=0; i < subject.length(); i++) {
			g.drawString( subject.getLine(i), x, y );
			y += linefeed;
		}

		//draw a line below the subject's text
		g.setColor( Color.orange );
		g.fillRect( x+40, y+20, 200, 5);

        }

	/**
	 *	Tries to open a about File using the fileDirectory
         *      and the filename of the about-file
	 *
	 *	param String 			filename of aboutfile
	 *	return Text			text found
	 *
	 *	@author Filip Van Rysselberghe
	 */
	private Text getAboutFile(String filename){
		try {
			URL fullFileLocation = new URL(fFileLocation, filename);

			return new Text(fullFileLocation);
		}
		catch ( MalformedURLException e) {
			return new Text();
		}
	}

	/**
         *      The size of the help frame (since it is'n resizable) 400x450
         *
	 *      @author Filip Van Rysselberghe
	 */
	public Dimension preferredSize(){
		return new Dimension(400,350);
        }

        /**
         * Initializes all components of the helpframe
         *
         * @author      Hans Stenten
         */
        private void jbInit() throws Exception {
                generalPanel.setLayout(borderLayout1);
                bodyPanel.setLayout(borderLayout2);
                this.setTitle("About - Algorithms in Action");
                this.setResizable(false);
                paddingBottom.setBackground(Color.white);
                aboutButton.setLabel("OK");
                aboutButton.addMouseListener(new java.awt.event.MouseAdapter() {
                        public void mouseClicked(MouseEvent e) {
                                aboutButton_mouseClicked(e);
                        }
                });
                this.add(generalPanel, BorderLayout.CENTER);
                generalPanel.add(bodyPanel, BorderLayout.CENTER);
                bodyPanel.add(fScroller, BorderLayout.CENTER);
                this.add(paddingBottom, BorderLayout.SOUTH);
                paddingBottom.add(aboutButton);
                pack();
                validate();
                show();
                showSubject("Algoritms in Action", getAboutFile("aia.about"));
        }

        void aboutButton_mouseClicked(MouseEvent e) {
                dispose();
        }
}

