package project;

/**
 *	This class is the basic class that provides the necessary methods for animating
 *	the sorting proces of the algorithm. Every SortAlgThread has one anim drawer and uses it to animate
 *	its steps.
 *
 *	@author			Hans Stenten
 *	@author			Filip Van Rysselberghe
 *	@version		1.10, 21/11/2000
 */
import java.awt.*;

public abstract class AlgAnimDrawer {
	protected AlgThread fThread;
        protected AlgAnimatorConfig fConfig;
        protected boolean fAnimate;

	/**
	 *
	 *	Constructor
	 *
	 *	@author			Hans Stenten
	 *	@param			thread	containing sort-thread, initializes fThread
         *      @param                  configuration contains the current configuration of the application
	 */
	public AlgAnimDrawer(AlgThread thread, AlgAnimatorConfig configuration) {
                System.err.println("AnimDrawer in");
		fThread = thread;
                System.err.println("1");
                fConfig = configuration;
                System.err.println("2");
                fAnimate = !fConfig.isCompare();
                System.err.println("animate");
                animate();
                System.err.println("AnimDrawer out");
      	}

	/**
	 *
	 *	The sorting-thread will be stopped for a while so that the user can try to
	 *	figure out what's really happening
	 *
	 *	@author	Filip Van Rysselberghe
	 */
	protected void sleep() throws InterruptedException{
		if (fConfig.isStep()){
			//when the thread has to evolve in steps the thread won't be put to sleep
			//but will be suspended until the user presses the step button one more
			//time
			fThread.suspend();
		}
		else{
			//in all other cases the thread will be put to sleep for a while. The time
			//that this has to last has been specified in the configuration.
			fThread.sleep(fThread.fConfig.getStepTime());
		}
	}

        /**
         *      Abstract function called by constructor to display initial data
         *
         *      @author Hans Stenten
         */
        public abstract void animate();
}
