package project;

/**
 *	This class contains the current configuration of the applet.
 *	It contains the URL where the applet is located (codeBase),
 *	the type of array to search / sort on (itemStyle) (random [0],
 *	ascending [1], descending [2] or equal [4]). The time between steps
 *	(stepTime).
 *	status and language aren't in use for the moment,
 *	but will be in the future.
 *	The AlgThread can get the setting from the class,
 *	and the user can set the settings in this class through the ConfigDialog
 *
 *	@author	Hans Stenten
 */

import java.net.URL;

public class AlgAnimatorConfig{
        public int height;
        public int width;
	private String fAlgorithmName;
	private int fItemStyle;
	private int fStepTime;
	private int fStatus;
        private int fSearchValue;
	private URL fCodeBase;
        private boolean fStep;
        private boolean fCompare;
        private boolean fRunning;


	/**
	 *	Constructor, gets the codeBase as parameter
	 *	all other variables are set to a default value
	 *	algorithm name	Bubblesort
	 *	item style		0 (random)
	 *	step time		100 milliseconds
	 *	status			0 (no use yet)
	 *	REMARK:
	 *	there is no standard constructor without parameter
	 *	since codeBase shouldn't be re-set during execution
	 *	only when initialized it should be set
	 *
	 *	@author		Hans Stenten
	 *	@param		base base URL
	 */
	public AlgAnimatorConfig(URL base){
		fAlgorithmName = "Bubblesort";
		fItemStyle = 0;
		fStepTime = 100;
		fStatus = 0;
		fCodeBase = base;
                fCompare = false;
	}

	/**
	 *	Sets the algorithm name to the name given as parameter
	 *
	 *	@author		Hans Stenten
	 *	@return		void
	 *	@param		name name of the algorithm
	 */
	public void setAlgorithmName(String name){
		fAlgorithmName = name;
	}

	/**
	 *	Returns the current algorithm name
	 *
	 *	@author		Hans Stenten
	 *	@return		String
	 */
	public String getAlgorithmName(){
		return fAlgorithmName;
	}

	/**
	 *	Sets the type of data in the array
	 *	0	random
	 *	1	ascending
	 *	2	descending
	 *	3	equal
	 *
	 *	@author		Hans Stenten
	 *	@return		void
	 *	@param		style   content of array by some style
	 */
	public void setItemStyle(int style){
		fItemStyle = style;
	}

	/**
	 *	Returns the current item style
	 *
	 *	@author		Hans Stenten
	 *	@return		int
	 */
	public int getItemStyle(){
		return fItemStyle;
	}

	/**
	 *	Sets the time between steps by the value given as parameter.
	 *	The time is in milliseconds
	 *
	 *	@author		Hans Stenten
	 *	@return		void
	 *	@param		time    set amount of time to wait between steps in milliseconds
	 */
	public void setStepTime(int time){
		fStepTime = time;
	}

	/**
	 *	Returns the time in millieseconds between two steps of the algorithm
	 *	@author		Hans Stenten
	 *	@return		int
	 */
	public int getStepTime(){
		return fStepTime;
	}

	/**
	 *	Returns the current status of the frame
	 *	used to determine which help / config dialog
	 *	should be showed
	 *	not used yet
	 *	@author		Hans Stenten
	 *	@return		int
	 */
	public int getStatus(){
		return 0;
	}

	/**
	 *	Returns the URL of the class of this applet
	 *	@author		Hans Stenten
	 *	@return		URL
	 */
	public URL getCodeBase(){
		return fCodeBase;
	}

        /**
	 *	Tells whether the algorithm is animated in a stepwise manner.
	 *
	 *	@author			Hans Stenten
	 *	@return			boolean
	 */
	public boolean isStep(){
		return fStep;
	}

	/**
	 *	Switches on the stepwise animation of the current algorithm.
	 *
	 *	@author			Hans Stenten
	 *	@param			step    wait after each step yes/no
	 */
	public void setStep(boolean step){
		fStep = step;
	}

        /**
	 *	Tells whether the algorithms are compared
	 *
	 *	@author			Hans Stenten
	 *	@return			boolean
	 */
	public boolean isCompare(){
		return fCompare;
	}

	/**
	 *	Set compare or not (animate algorithms seperatly or not)
	 *
	 *	@author			Hans Stenten
	 *	@param			compare
	 */
	public void setCompare(boolean compare){
		fCompare = compare;
	}

        /**
         *      Return true if the algorithm is a search algorithm
         *
         *      @author                 Hans Stenten
         */
        public boolean isSearch(){
                if (fAlgorithmName.indexOf("search") > -1) return true;
                if (fAlgorithmName.indexOf("search") == -1) return false;
                return false;
        }

        /**
         *      Return the search value
         *
         *      @author                 Hans Stenten
         */
        public int getSearchValue(){
                return fSearchValue;
        }

        /**
         *      Set the search value
         *
         *      @author                 Hans Stenten
         */
        public void setSearchValue(int search){
                fSearchValue = search;
        }

        /**
         * Returns true if algorithm has ended
         *
         * @author Hans Stenten
         */
        public boolean isEnded(){
                return !fRunning;
        }

        /**
         * Sets whether the algorithm is running
         *
         * @author Hans Stenten
         */
        public void setRunning(boolean run){
                fRunning = run;
        }
}
