package project;

import java.awt.*;
import java.awt.event.*;

/**
 *	Dialog allows user to set certain variables in the animation of the
 *      algorithm
 *
 *	@author	Hans Stenten
 */

public class AlgAnimatorDialog extends Dialog {
        Panel contentPanel = new Panel();
        BorderLayout borderLayout1 = new BorderLayout();
        Panel heading = new Panel();
        Panel center = new Panel();
        Panel lower = new Panel();
        Button Apply = new Button();
        Button Cancel = new Button();
        Label dialogTitle = new Label();
        AlgAnimatorConfig fConfig;
        BorderLayout borderLayout2 = new BorderLayout();
        Label stepLabel = new Label();
        Label styleLabel = new Label();
        Choice styleChoice = new Choice();
        Scrollbar stepScroll = new Scrollbar();
        TextField stepField = new TextField();

        /**
         *      Makes dialog, sets its title and links to the configuration of the applet
         *
         *      @see AlgAnimatorConfig
         *      @author Hans Stenten
         */
        public AlgAnimatorDialog(Frame frame, String title, AlgAnimatorConfig config) {
                super(frame, title, false);
                fConfig = config;
                enableEvents(AWTEvent.WINDOW_EVENT_MASK);
                try {
                        jbInit();
                        add(contentPanel);
                        pack();
                        setSize(500,300);
                        setResizable(false);
                }
                catch(Exception ex) {
                        ex.printStackTrace();
                }
        }

        /**
         *      Initializes the components
         *
         *      @author Hans Stenten
         */
        private void jbInit() throws Exception {
                contentPanel.setLayout(borderLayout1);
                Apply.setLabel("Apply");
                Cancel.setLabel("Cancel");
                dialogTitle.setText("Configure "+fConfig.getAlgorithmName());
                dialogTitle.setBackground(Color.lightGray);
                center.setLayout(borderLayout2);
                stepLabel.setText("Step Time");
                stepLabel.setBackground(Color.lightGray);
                styleLabel.setText("Content of the array");
                styleLabel.setBackground(Color.lightGray);
                stepScroll.setBlockIncrement(100);
                stepScroll.setMaximum(1010);
                stepScroll.setOrientation(0);
                stepScroll.setUnitIncrement(10);
                stepScroll.setValue(100);
                stepField.setBackground(Color.lightGray);
                stepField.setEditable(false);
                stepField.setText("100");
                // sort
                if (fConfig.getAlgorithmName().indexOf("sort") >= 0){
                        styleChoice.add("Random");
	        	styleChoice.add("Ascending");
		        styleChoice.add("Descending");
        		styleChoice.add("Equal");
                        sortInit();
                }
                // search
                else if (fConfig.getAlgorithmName().indexOf("search") >= 0){
                        styleChoice.add("Ascending");
                        searchInit();
                }
                contentPanel.add(heading, BorderLayout.NORTH);
                heading.add(dialogTitle, null);
                contentPanel.add(lower, BorderLayout.SOUTH);
                lower.add(Apply, null);
                lower.add(Cancel, null);
                contentPanel.add(center, BorderLayout.CENTER);

                // ACTIONS
                stepScroll.addAdjustmentListener(new AdjustmentAdapter(){
			public void adjustmentValueChanged(java.awt.event.AdjustmentEvent e){
				stepField.setText(""+e.getValue());
			}
		});

                Apply.addMouseListener(new MouseAdapter(){
			public void mouseClicked(MouseEvent e){
				fConfig.setItemStyle(styleChoice.getSelectedIndex());
				fConfig.setStepTime(stepScroll.getValue());
				System.err.println("configuration changed:");
				System.err.println(" - step time  : " + fConfig.getStepTime());
                                System.err.println(" - array style  : " + fConfig.getItemStyle());
				setVisible(false);
			}
		});

                Cancel.addMouseListener(new MouseAdapter(){
			public void mouseClicked(MouseEvent e){
				setVisible(false);
			}
		});

        }

        /**
         *      Sets the components for a sort function
         *
         *      @author Hans Stenten
         */
        void sortInit() throws Exception{
                GridBagLayout gb = new GridBagLayout();
		GridBagConstraints gc = new GridBagConstraints();
                center.setLayout(gb);
                gc.fill = GridBagConstraints.HORIZONTAL;
                // 1st row 1st column : array label
                gc.anchor = GridBagConstraints.NORTH;
                gc.gridx = 0;
                gc.gridy = 0;
                gc.weightx = 1.0;
                gc.weighty = 0.0;
                gb.setConstraints( styleLabel, gc );
		center.add( styleLabel );
                // 1st row 2nd column : array combo
                gc.gridwidth = GridBagConstraints.REMAINDER;
                gc.gridx = 1;
                gc.gridy = 0;
                gc.weightx = 2.0;
                gc.gridwidth = 2;
                gb.setConstraints( styleChoice, gc );
                center.add( styleChoice );
                // 2nd row 1st column : step label
                gc.gridwidth = GridBagConstraints.RELATIVE;
                gc.gridwidth = 1;
                gc.weightx = 1.0;
                gc.weighty = 0.0;
                gc.gridx = 0;
                gc.gridy = 1;
                gb.setConstraints( stepLabel, gc );
                center.add( stepLabel );
                // 2nd row 2nd column : step scrollbar
                gc.gridx = 1;
                gc.gridy = 1;
                gb.setConstraints( stepScroll, gc );
                center.add( stepScroll );
                // 2nd row 3th column : step field
                gc.gridwidth = GridBagConstraints.REMAINDER;
                gc.weightx = 0.0;
                gc.gridx = 2;
                gc.gridy = 1;
                gb.setConstraints( stepField, gc );
                center.add( stepField );
        }

        /**
         *      Sets the components for a search function
         *
         *      @author Hans Stenten
         */
        void searchInit(){
                GridBagLayout gb = new GridBagLayout();
		GridBagConstraints gc = new GridBagConstraints();
                center.setLayout(gb);
                gc.fill = GridBagConstraints.HORIZONTAL;
                // 1nd row 1st column : step label
                gc.gridwidth = GridBagConstraints.RELATIVE;
                gc.gridwidth = 1;
                gc.weightx = 1.0;
                gc.weighty = 0.0;
                gc.gridx = 0;
                gc.gridy = 1;
                gb.setConstraints( stepLabel, gc );
                center.add( stepLabel );
                // 1nd row 2nd column : step scrollbar
                gc.gridx = 1;
                gc.gridy = 1;
                gb.setConstraints( stepScroll, gc );
                center.add( stepScroll );
                // 1nd row 3th column : step field
                gc.gridwidth = GridBagConstraints.REMAINDER;
                gc.weightx = 0.0;
                gc.gridx = 2;
                gc.gridy = 1;
                gb.setConstraints( stepField, gc );
                center.add( stepField );
        }

        protected void processWindowEvent(WindowEvent e) {
                if (e.getID() == WindowEvent.WINDOW_CLOSING) {
                        cancel();
                }
                super.processWindowEvent(e);
        }
        void cancel() {
                dispose();
        }
}