package project;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.io.*;
import java.net.*;
import java.lang.*;

/**
 *
 *	The main interface class and extension of frame, this class contains the different panels
 *  on which the animation takes place.
 *  It also makes, starts and stops the algorithm-threads via specific methods.
 *
 *	@author	Hans Stenten
 *	@author	Filip Van Rysselberghe
 */

public class AlgAnimatorFrame extends Frame {
        //Parent applet
        private AlgAnimatorApplet fParentApp;

        //configuration
        private AlgAnimatorConfig	fConfig;

        //thread that executes the actual algorithm
        private AlgThread       	fAlgThread;

        //
        private ImagesThread    	fImages;

        // GUI ELEMENTS
        BorderLayout borderLayout1 = new BorderLayout();
        Panel fSearchsort = new Panel();
        Panel fWelcome = new Panel();
        Panel fCompare = new Panel();
        BorderLayout borderLayout2 = new BorderLayout();
        ImagePanel fAnimationPanel = new ImagePanel();
        GridLayout gridLayout1 = new GridLayout();
        Panel panel1 = new Panel();
        ImagePanel fSourcePanel = new ImagePanel();
        StatusPanel fStatusPanel = new StatusPanel();
        Panel right = new Panel();
        BorderLayout borderLayout3 = new BorderLayout();
        ControlPanel fControlPanel;
        BorderLayout borderLayout4 = new BorderLayout();
        Label fWelcomeLabel = new Label();

        //MENU
        MenuBar menuBar1 = new MenuBar();
        Menu fileMenu = new Menu();
        Menu algMenu  = new Menu();
        Menu helpMenu = new Menu();
        MenuItem aboutItem = new MenuItem();
        MenuItem helpItem = new MenuItem();
        MenuItem menuItem5 = new MenuItem();

        //Menu with sorting algorithms
        Menu sortMenu = new Menu();
        MenuItem bsort = new MenuItem();	//bubblesort
        MenuItem isort = new MenuItem();	//insertionsort
        MenuItem qsort = new MenuItem();	//quicksort

        //Menu with searching algorithms
        Menu searchMenu = new Menu();
        MenuItem bsearch = new MenuItem();	//binary search
        MenuItem ssearch = new MenuItem();	//sequential search

	//Menu with IPC(inter process communication) problems
	Menu ipcMenu = new Menu();
	MenuItem dinPhil = new MenuItem();	//dining philosophers

        /**
         * Constructor of the AlgAnimator frame class, taking the containing applet as parameter.<br>
         * Sets the general configuration of the applet, this configuration can be changed by the user later.<br>
         * Sets the frame title, the menubar and a welcome screen.
         *
         * @param 	parentApp 					the parent applet
         * @author 	Hans Stenten
         * @author 	Filip Van Rysselberghe
         */
        public AlgAnimatorFrame(AlgAnimatorApplet parent) {
                fParentApp = parent;

                fConfig = new AlgAnimatorConfig(fParentApp.getCodeBase());
                fControlPanel = new ControlPanel(this, fConfig);

                fAlgThread = null;
                fImages = null;

                try {
                        jbInit();
                        setWelcome();
                        setTitle("Welcome to Algorithms in Action");
                }
                catch(Exception e) {
                        e.printStackTrace();
                }

                System.err.println(getSize().width+"x"+getSize().height);

                //height - 34 because we have to take into account the size
		//of the menubar and the title rule
		fConfig.height = getSize().height - 14;
                fConfig.width = getSize().width;
        }

        /**
         * Initiates all components of the frame
         *
         * @author Hans Stenten
         */
        private void jbInit() throws Exception {
                fWelcomeLabel.setFont(new java.awt.Font("Courier", 1, 36));
                fWelcomeLabel.setText("Algorithms In Action");

                fWelcome.setLayout(borderLayout4);
                fWelcome.setBackground(Color.gray);

                right.setLayout(borderLayout3);
                panel1.setLayout(gridLayout1);
                fSearchsort.setLayout(borderLayout2);

                this.setLayout(borderLayout1);
                this.setBackground(Color.lightGray);
                fWelcome.setFont(new java.awt.Font("Dialog", 0, 24));



                this.addWindowListener(new java.awt.event.WindowAdapter() {
                        public void windowClosing(WindowEvent e) {
                                this_windowClosing(e);
                        }
                });

                this.addComponentListener(new java.awt.event.ComponentAdapter() {
                        public void componentResized(ComponentEvent e) {
                                this_componentResized(e);
                        }
                });

                this.setMenuBar(menuBar1);

                fileMenu.setLabel("File");
                algMenu.setLabel("Algorithms");
                helpMenu.setLabel("Help");

                aboutItem.setLabel("About");
                aboutItem.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                aboutItem_actionPerformed(e);
                        }
                });
                helpItem.setLabel("Help");
                helpItem.addActionListener(new java.awt.event.ActionListener() {
                	public void actionPerformed(ActionEvent e){
                		helpItem_actionPerformed(e);
                	}
                });

                menuItem5.setLabel("Exit");
                menuItem5.addActionListener(new java.awt.event.ActionListener() {
                	public void actionPerformed(ActionEvent e){
                		this_windowClosing(e);
                	}
                });

                //sorting menu with all the possible sort-options and their event-handling
                sortMenu.setLabel("Sorting Algorithms");

                bsort.setLabel("Bubblesort");
                bsort.setActionCommand("Bubblesort");
                bsort.addActionListener(new java.awt.event.ActionListener() {
                		public void actionPerformed(ActionEvent e) {
                                bsort_actionPerformed(e);
                        }
                });

                isort.setLabel("Insertionsort");
                isort.setActionCommand("Insertionsort");
                isort.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                isort_actionPerformed(e);
                        }
                });
                qsort.setLabel("Quicksort");
                qsort.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                qsort_actionPerformed(e);
                        }
                });

                //search menu with all the possible search-options and their event-handling
                searchMenu.setLabel("Searching Algorithms");

                bsearch.setLabel("Binarysearch");
                bsearch.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                bsearch_actionPerformed(e);
                        }
                });

                ssearch.setLabel("Sequentialsearch");
                ssearch.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                                ssearch_actionPerformed(e);
                        }
                });

		ipcMenu.setLabel("IPC problems");

		dinPhil.setLabel("Dining Philosophers");
		dinPhil.setActionCommand("DiningPhilosophers");
		dinPhil.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e){
				dinPhil_actionPerformed(e);
			}
		});

                fSearchsort.add(panel1, BorderLayout.CENTER);
                //panel1.add(fAnimationPanel, null);
                //panel1.add(right, null);
                panel1.add(fAnimationPanel, BorderLayout.WEST);
                panel1.add(right, BorderLayout.EAST);

                right.add(fStatusPanel, BorderLayout.SOUTH);
                right.add(fSourcePanel, BorderLayout.CENTER);
                fSearchsort.add(fControlPanel, BorderLayout.SOUTH);

                fWelcome.add(fWelcomeLabel, BorderLayout.SOUTH);

                menuBar1.add(fileMenu);
                menuBar1.add(algMenu);
                menuBar1.add(helpMenu);

                algMenu.add(sortMenu);
                algMenu.add(searchMenu);
		algMenu.add(ipcMenu);

                helpMenu.add(aboutItem);
                helpMenu.add(helpItem);
                fileMenu.add(menuItem5);

                sortMenu.add(bsort);
                sortMenu.add(isort);
                sortMenu.add(qsort);

                searchMenu.add(bsearch);
                searchMenu.add(ssearch);

		ipcMenu.add(dinPhil);
        }

        /**
         * Displays the basic welcome screen
         *
         * @author Hans Stenten
         */
        private void setWelcome(){
                add(fWelcome, BorderLayout.CENTER);
                validate();
                pack();
                show();
        }

        /**
         * Displays the configuration to animate search and sort algorithms
         *
         * @author Hans Stenten
         */
        private void setSearchSort(){
                add(fSearchsort, BorderLayout.CENTER);
                Panel[] panels = new Panel[]{ fAnimationPanel, fSourcePanel, fStatusPanel, fControlPanel};
                fImages = new ImagesThread(panels, fConfig);
                validate();
                pack();
                show();
        }

	/**
         * Clears panel, so a new configuration can be added without problems
         *
         * @author Hans Stenten
         */
        private void clearPanel(){
                removeAll();
        }

        /**
         * This method makes a new algorithm thread, by creating a new one, or killing an old one and recreating a new one.<br>
         * It creates the correct thread by using the configuration class<br>
         *
         * @param none
         * @return void
         * @author Hans Stenten
         * @see AlgAnimatorConfig
         */
	private void makeAlgorithm(){
        	if ( fAlgThread == null ) {
			try {
				//we create the thread class dynamicly using the
				//provided classname.
				String classname = "project." + fConfig.getAlgorithmName() + "AlgThread";
                                System.err.println("classname : "+classname);
				Class algClass = Class.forName(classname);
                                System.err.println("forName ");
				Class[] params = new Class[] {fImages.getClass(), fConfig.getClass()};
                                System.err.println("class params "+classname);
				Object[] params2 = new Object[] {fImages, fConfig};
                                System.err.println("object params"+classname);
				Object o = (algClass.getConstructor(params)).newInstance(params2);
                                System.err.println("new object"+classname);
				fAlgThread = (AlgThread)o;
			}
			catch ( Exception e ) {
				System.err.println("can not create class");
				System.err.println(e.getMessage());
			}
		}
		else{
			fAlgThread = null;
			makeAlgorithm();
		}
		setTitle("Algorithms in Action : " +fConfig.getAlgorithmName());
	}

        /**
         * Starts an algorithm thread, by making one if needed, and start / resume it.<br>
         * By default the thread is set to "run" and not to "step", but this can be changed by the user through the configuration<br>
         *
         * @param none
         * @return void
         * @author Hans Stenten
         * @see AlgAnimatorConfig
         */
	public void startAlgorithm(){
		if ( fAlgThread == null ) {
			makeAlgorithm();
		}
		fConfig.setStep(false);
		if (fAlgThread.isAlive()){
			fAlgThread.resume();
		}
		else{
			fAlgThread.start();
		}
	}

       /**
        * Stops the algorithm thread and sets it to null
        *
        * @author Hans Stenten
        * @param none
        * @return void
        */
	public void stopAlgorithm(){
		if ( fAlgThread != null && fAlgThread.isAlive() ) {
			fAlgThread.stopAlgorithm();
		}
		fAlgThread = null;
	}

        /**
         * Sets the thread to step, which means the algorithm will "halt" after each step
         *
         * @author Hans Stenten
         * @return void
         * @param none
         */
	public void stepAlgorithm(){
		if (fAlgThread == null){
			makeAlgorithm();
		}
		fConfig.setStep(true);
		if (fAlgThread.isAlive()){
			fAlgThread.resume();
		}
		else{
			fAlgThread.start();
		}
	}

        /**
         * When frame is resized, new height and width are needed for a smoother
         * animation of the algorithm
         *
         * @author Hans Stenten
         */
        private void this_componentResized(ComponentEvent e) {
                //the factor -14 is for taking into account the
		//size of menubar and window-titlebar
		fConfig.height = getSize().height - 14;
        fConfig.width = getSize().width;
        }

        /**
         * Stop all threads when frame is closed
         *
         * @author Hans Stenten
         */
        private void this_windowClosing(AWTEvent e) {
                if (fAlgThread != null) fAlgThread.stop();
                if (fImages != null) fImages.stop();

                fParentApp.fStartButton.setEnabled(true);
                setVisible(false);
        }

        /**
         * Handles event fired by frame
         *
         * @return boolean
         * @author Hans Stenten
         * @author Filip Van Rysselberghe
         */
        public boolean handleEvent(Event e ) {

		if ( e.id == Event.WINDOW_DESTROY) {
			dispose();
			fParentApp.fStartButton.enable();
		}
		return super.handleEvent( e );
	}

        /**
         * Sets the preferred size of the frame
         *
         * @return Dimension
         * @author Hans Stenten
         * @author Filip Van Rysselberghe
         */
        public Dimension preferredSize() {
		return new Dimension(800, 600);
	}

        /**
         * Sets applet to animate bubblesort
         *
         * @author Hans Stenten
         */
        private void bsort_actionPerformed(ActionEvent e) {
                fConfig.setAlgorithmName(e.getActionCommand());
                clearPanel();
                setSearchSort();
        }

        /**
         * Sets applet to animate insertionsort
         *
         * @author Hans Stenten
         */
        private void isort_actionPerformed(ActionEvent e) {
                fConfig.setAlgorithmName(e.getActionCommand());
                clearPanel();
                setSearchSort();
        }

        /**
         * Sets applet to animate quicksort
         *
         * @author Hans Stenten
         */
        private void qsort_actionPerformed(ActionEvent e) {
                fConfig.setAlgorithmName(e.getActionCommand());
                clearPanel();
                setSearchSort();
        }

        /**
         * Sets applet to animate binarysearch
         *
         * @author Hans Stenten
         */
        private void bsearch_actionPerformed(ActionEvent e) {
                fConfig.setAlgorithmName(e.getActionCommand());
                clearPanel();
                setSearchSort();
        }

        /**
         * Sets applet to animate sequentialsearch
         *
         * @author Hans Stenten
         */
        private void ssearch_actionPerformed(ActionEvent e) {
                fConfig.setAlgorithmName(e.getActionCommand());
                clearPanel();
                setSearchSort();
        }

	/**
	 * Sets the applet to animate Dining Philosiophers
	 *
	 * @author Filip Van Rysselberghe
	 */
	private void dinPhil_actionPerformed(ActionEvent e) {
		fConfig.setAlgorithmName(e.getActionCommand() );
		clearPanel();

		//tijdelijk nog zo...
		setSearchSort();
	}

        /**
         * Fires up the HelpFrame
         *
         * @author Filip Van Rysselberghe
         */
        private void helpItem_actionPerformed(ActionEvent e) {
		HelpFrame helpFr = new HelpFrame(fConfig.getCodeBase());
	}

        /**
         * Fires up the AboutFrame
         *
         * @author Hans Stenten
         */
        private void aboutItem_actionPerformed(ActionEvent e) {
                AboutFrame aboutFr = new AboutFrame(fConfig.getCodeBase());
        }
}
