package project;

/**
 *	This class is the basic class that provides the necessary methods for animating
 *	the sorting proces of the algorithm. Every SortAlgThread has one anim drawer and uses it to animate
 *	its steps.
 *
 *	@author			Hans Stenten
 *	@author			Filip Van Rysselberghe
 *	@version		1.10, 21/11/2000
 */

import java.awt.Graphics;
import java.awt.Image;
import java.awt.Color;
import java.awt.Font;


public class AlgSearchAnimDrawer extends AlgAnimDrawer {
        /**
	 *
	 *	Constructor
	 *
	 *	@author			Hans Stenten
	 *	@param			thread	containing sort-thread, initializes fThread
         *      @param                  configuration contains the current configuration of the application
	 */
	public AlgSearchAnimDrawer(AlgThread thread, AlgAnimatorConfig configuration) {
                super(thread, configuration);
      	}

        /**
	 *
	 *	This animate function is used for animating
	 *	the bars that represent the data-array
	 *
	 *	@author 	Hans Stenten
	 *	@author 	Filip Van Rysselberghe
	 *
	 *	@param		active	the element being compared
	 */
	public void animate(int active) throws InterruptedException{
                if (fAnimate){
        		Image 		image 	= fThread.fImages.getAnimationImage();
	        	Graphics	g 	= image.getGraphics();

		        int 	width  = fThread.fImages.getAnimationDimension().width,
			        height = fThread.fImages.getAnimationDimension().height;

        		g.setColor( Color.lightGray );
	        	g.fillRect(0,0, width, height);

		        int maxValue = ((SearchAlgThread)fThread).fItemsToSearch.getMaxValue();
        		int[] values = ((SearchAlgThread)fThread).fItemsToSearch.toArray();

	        	if ( values.length <= 0 ) {
		        	//throw exception
			        return;
        		}

	        	//spacing will define the space between two lines. We have chosen
		        //to make the gaps between the lines as high as the lines. This spacing
        		//doesn't use the maximum of available space but half of the available
	        	//space
		        int     spacing = (int)(height/(2*values.length)),
        			unit	= (int)( (0.125)*width ),
	        		x 		= unit,
		        	heightStripe	= spacing,
			        scale;

        		if ( maxValue == 0 ) {
        			scale = 1;
	        	}
        		else {
	        		scale = (width - unit) / maxValue;
        		}

	        	for (int i = 0; i < values.length; ++i){
		        	int y 			= ( 2 * i ) * spacing,
			        	widthStripe = values[i] * scale;

        			if ( maxValue < 0 ) {
	        			widthStripe = (-1) * widthStripe;
		        	}
        			drawStripe(g, x, y, widthStripe, heightStripe, active==i);
	        	}
        		drawArray(g, 0, 0, unit-4, spacing, active, values);
	        	//do the actual paint of the changed image
        		fThread.fImages.paintAnimation(image);
        		this.sleep();
        	}
        }

	/**
	 *
	 *	Draws the stripes that represent the array values. It only uses dynamic
	 *	data. The use of the available place is maximized.
	 *
	 *	@author			Hans Stenten
	 *	@author			Filip Van Rysselberghe
	 *	@param			g		element to draw on
	 *	@param			x			x-coordinate where stripe starts
	 *	@param			y			y-coordinate where stripe starts
	 *	@param			width		width of the stripe
	 *	@param			height		height of the stripe
	 *	@param			active	        true if element is the element of the array the algorithm is working on
	 *
	 *	@see drawArray
	 */
	private void drawStripe(Graphics g, int x, int y, int width, int height, boolean active){
		if (active){
			g.setColor(Color.red);
		}
		else{
			g.setColor(Color.blue);
		}
		g.fillRect(x, y, width, height);
	}

	/**
	 *
	 *	new version of draw array. This one only uses dynamic values.
	 *
	 *	@author			Hans Stenten
	 *	@author			Filip Van Rysselberghe
	 *
	 *	@param			g		element to draw on
	 *	@param			x		x-coordinate where started
	 *	@param			y		y-coordinate where started
	 *	@param			width		width of the stripes
	 *	@param			height		height of the stripes
         *      @param                  active          the active element
	 *	@param			a		array drawn
	 */
	private void drawArray(Graphics g, int x, int y, int width, int height, int active, int[] a){
		for(int i=0; i<a.length; ++i){
			int yRelative = (i * (2 * height) + y);
                        if (active == i){
        			g.setColor(Color.red);
	        		g.fillRect(x, yRelative, width, height);
                        }
			g.setColor(Color.black);
			g.drawRect(x, yRelative, width, height);
			g.setFont( new Font("Helvetica", Font.PLAIN, height) );
			g.drawString( "" + a[i], x + (int)(0.25 * width) ,(i * (2 * height) + y + height) );
		}
	}

	/**
	 *      Draws Initial data
	 *
	 *	@author			Hans Stenten
	 */
        public void animate(){
                if (fAnimate){
               		Image 		image 	= fThread.fImages.getAnimationImage();
	        	Graphics	g 	= image.getGraphics();

		        int 	width  = fThread.fImages.getAnimationDimension().width,
                                height = fThread.fImages.getAnimationDimension().height;

	        	g.setColor( Color.lightGray );
		        g.fillRect(0,0, width, height);

        		int maxValue = ((SearchAlgThread)fThread).fItemsToSearch.getMaxValue();
	        	int[] values = ((SearchAlgThread)fThread).fItemsToSearch.toArray();

        		if ( values.length <= 0 ) {
        			//throw exception
	        		return;
        		}

	        	//spacing will define the space between two lines. We have chosen
		        //to make the gaps between the lines as high as the lines. This spacing
        		//doesn't use the maximum of available space but half of the available
	        	//space
        		int     spacing = (int)(height/(2*values.length)),
	        		unit	= (int)( (0.125)*width ),
		        	x 		= unit,
			        heightStripe	= spacing,
        			scale;
        		if ( maxValue == 0 ) {
	        		scale = 1;
		        }
        		else {
	        		scale = (width - unit) / maxValue;
		        }

        		for (int i = 0; i < values.length; ++i){
	        		int y 			= ( 2 * i ) * spacing,
		        		widthStripe = values[i] * scale;

			        if ( maxValue < 0 ) {
				        widthStripe = (-1) * widthStripe;
        			}
	        		drawStripe(g, x, y, widthStripe, heightStripe, false);
        		}
                	drawArray(g, 0, 0, unit-4, spacing, -1, values);
        		//do the actual paint of the changed image
	        	fThread.fImages.paintAnimation(image);
                }
        }
}