package project;

/**
 *	This class is the basic class that provides the necessary methods for animating
 *	the sorting proces of the algorithm. Every SortAlgThread has one anim drawer and uses it to animate
 *	its steps.
 *
 *	@author			Hans Stenten
 *	@author			Filip Van Rysselberghe
 *	@version		1.10, 21/11/2000
 */

import java.awt.Graphics;
import java.awt.Image;
import java.awt.Color;
import java.awt.Font;


public class AlgSortAnimDrawer extends AlgAnimDrawer {
        private boolean fPivot = false;

        /**
	 *
	 *	Constructor
	 *
	 *	@author			Hans Stenten
	 *	@param			thread	containing sort-thread, initializes fThread
         *      @param                  configuration contains the current configuration of the application
	 */
	public AlgSortAnimDrawer(AlgThread thread, AlgAnimatorConfig configuration) {
                super(thread, configuration);
                System.err.println("SortAnimDrawer in");
      	}

        /**
	 *
	 *	This second animate function has one extra parameter and is used for animating
	 *	the bars that represent the data-array been animated.
	 *
	 *	@author 	Hans Stenten
	 *	@author 	Filip Van Rysselberghe
	 *
	 *	@param		first			the first element (or only if no swap)
	 *	@param		second			the second element of the swap or not relevant
	 *	@param		swapped	        	animate a swap or not
	 */
	public void animate(int first, int second, boolean swapped) throws InterruptedException{
                if(fAnimate){
        		Image 		image 	= fThread.fImages.getAnimationImage();
	        	Graphics	g 	= image.getGraphics();

        		// width - 4 to take the window border into account
                        // height - 80 to take the fixed height of the controlpanel into account
		        int 	width = fConfig.width/2 - 4,
                                height = fConfig.height-90;

	        	g.setColor( Color.lightGray );
		        g.fillRect(0,0, width, height);

        		int maxValue = ((SortAlgThread)fThread).fItemsToSort.getMaxValue();
	        	int[] values = ((SortAlgThread)fThread).fItemsToSort.toArray();

		        if ( values.length <= 0 ) {
        			//throw exception
	        		return;
		        }

	        	//spacing will define the space between two lines. We have chosen
		        //to make the gaps between the lines as high as the lines. This spacing
        		//doesn't use the maximum of available space but half of the available
	        	//space
        		int spacing     = (int)(height/(2*values.length)),
	        	    unit	= (int)( (0.125)*width ),
                            x 		= unit,
			    heightStripe= spacing,scale;


        		if ( maxValue == 0 ) {
	        		scale = 1;
		        }
        		else {
	        		scale = (width - unit) / maxValue;
		        }

        		for (int i = 0; i < values.length; ++i){
	        		int y 			= ( 2 * i ) * spacing,
		        		widthStripe = values[i] * scale;

			        if ( maxValue < 0 ) {
				        widthStripe = (-1) * widthStripe;
        			}

	        		drawStripe(g, x, y, widthStripe, heightStripe, second==i);
		        }

        		drawArray(g, 0, 0, unit-4, spacing, swapped, first, second, values);

	        	//do the actual paint of the changed image
		        fThread.fImages.paintAnimation(image);

        		this.sleep();
                }
	}

        /**
	 *
	 *	This second animate function has one extra parameter and is used for animating
	 *	the bars that represent the data-array been animated.
	 *
	 *	@author 	Hans Stenten
	 *	@author 	Filip Van Rysselberghe
	 *
	 *	@param		first			the first element (or only if no swap)
	 *	@param		second			the second element of the swap or not relevant
         *      @param          middle                  the pivot element
	 *	@param		swapped 		animate a swap or not
	 */
	public void animate(int first, int second, int middle, boolean swapped) throws InterruptedException{
                if(fAnimate){
        		Image 		image 	= fThread.fImages.getAnimationImage();
	        	Graphics	g 	= image.getGraphics();

        		// width - 4 to take the window border into account
                        // height - 80 to take the fixed height of the controlpanel into account
		        int 	width = fConfig.width/2 - 4,
                                height = fConfig.height-90;

	        	g.setColor( Color.lightGray );
		        g.fillRect(0,0, width, height);

        		int maxValue = ((SortAlgThread)fThread).fItemsToSort.getMaxValue();
	        	int[] values = ((SortAlgThread)fThread).fItemsToSort.toArray();

		        if ( values.length <= 0 ) {
        			//throw exception
	        		return;
		        }

	        	//spacing will define the space between two lines. We have chosen
		        //to make the gaps between the lines as high as the lines. This spacing
        		//doesn't use the maximum of available space but half of the available
	        	//space
        		int spacing     = (int)(height/(2*values.length)),
	        	    unit	= (int)( (0.125)*width ),
                            x 		= unit,
			    heightStripe= spacing,scale;


        		if ( maxValue == 0 ) {
	        		scale = 1;
		        }
        		else {
	        		scale = (width - unit) / maxValue;
		        }

        		for (int i = 0; i < values.length; ++i){
	        		int y 			= ( 2 * i ) * spacing,
		        		widthStripe = values[i] * scale;

			        if ( maxValue < 0 ) {
				        widthStripe = (-1) * widthStripe;
        			}
                                if (middle == i) fPivot = true;
	        		drawStripe(g, x, y, widthStripe, heightStripe, second==i);
                                if (middle == i) fPivot = false;
		        }

        		drawArray(g, 0, 0, unit-4, spacing, swapped, first, second, values);

	        	//do the actual paint of the changed image
		        fThread.fImages.paintAnimation(image);

        		this.sleep();
                }
	}


	/**
	 *
	 *	Draws the stripes that represent the array values. It only uses dynamic
	 *	data. The use of the available place is maximized.
	 *
	 *	@author			Hans Stenten
	 *	@author			Filip Van Rysselberghe
	 *	@param			Graphics g		element to draw on
	 *	@param			int x			x-coordinate where stripe starts
	 *	@param			int y			y-coordinate where stripe starts
	 *	@param			int width		width of the stripe
	 *	@param			int height		height of the stripe
	 *	@param			boolean active	        true if element is the element of the array the algorithm
	 *									is working on
	 *
	 *	@see drawArray
	 */
	private void drawStripe(Graphics g, int x, int y, int width, int height, boolean active){
                if (fAnimate){
        		if (active){
	        		g.setColor(Color.red);
		        }
        		else if (!active && fPivot){
	        		g.setColor(Color.green);
		        }
                        else{
                                g.setColor(Color.blue);
                        }

        		g.fillRect(x, y, width, height);
                }
	}

	/**
	 *
	 *	new version of draw array. This one only uses dynamic values.
	 *
	 *	@author			Hans Stenten
	 *	@author			Filip Van Rysselberghe
	 *
	 *	@param			Graphics g		element to draw on
	 *	@param			int x			x-coordinate where started
	 *	@param			int y			y-coordinate where started
	 *	@param			int width		width of the stripes
	 *	@param			int height		height of the stripes
	 *	@param			boolean swap	swapped or not
	 *	@param			int active1		first active element
	 *	@param			int active2		second active element
	 *	@param			int[] a			array drawn
	 *	@return			void
	 */
	private void drawArray(Graphics g, int x, int y, int width, int height, boolean swap, int active1, int active2, int[] a){
                if (fAnimate){
        		for(int i=0; i<a.length; ++i){
	        		int yRelative = (i * (2 * height) + y);

	        		if (active2 == i && swap){
		        		g.setColor(Color.red);
			        	g.fillRect(x, yRelative, width, height);
        			}
        			else if (active1 == i && swap){
	        			g.setColor(Color.red);
		        		g.fillRect(x, yRelative, width, height);
                                        drawSwap(g, x, yRelative - 2, width + 2, ((active2 - active1 + 1)*(2*height)));
	        		}
        			else if (active1 == i){
	        			g.setColor(Color.blue);
		        		g.fillRect(x, yRelative, width, height);
			        }
        			g.setColor(Color.black);
	        		g.drawRect(x, yRelative, width, height);

		        	g.setFont( new Font("Helvetica", Font.PLAIN, height) );
			        g.drawString( "" + a[i], x + (int)(0.25 * width) ,(i * (2 * height) + y + height) );
        		}
                }
	}

	/**
	 *
	 *	private void drawSwap(Graphics g, int element1, int element2, int width, int spacing)
	 *	draws a rectangle around the swapped elements
	 *
	 *	@author			Hans Stenten
	 *	@param			Graphics g
	 *	@param			int x
	 *	@param			int y
	 *	@param			int width
	 *	@param			int height
	 *	@return			void
	 */
	private void drawSwap(Graphics g, int x, int y, int width, int height){
		g.setColor(Color.orange);
		g.drawRect(x, y, width, height);
	}

	/**
	 *      Draws Initial data
	 *
	 *	@author			Hans Stenten
	 */
        public void animate(){
                if (fAnimate){
                        System.err.println("anim animate()");
               		Image 		image 	= fThread.fImages.getAnimationImage();
	        	Graphics	g 	= image.getGraphics();
                        // width - 4 to take the window border into account
                        // height - 80 to take the fixed height of the controlpanel into account
		        int 	width = fConfig.width/2 - 4,
                                height = fConfig.height-80;

        		g.setColor( Color.lightGray );
	        	g.fillRect(0,0, width, height);

		        int maxValue = ((SortAlgThread)fThread).fItemsToSort.getMaxValue();
        		int[] values = ((SortAlgThread)fThread).fItemsToSort.toArray();

	        	if ( values.length <= 0 ) {
		        	//throw exception
        			return;
	        	}

		        //spacing will define the space between two lines. We have chosen
        		//to make the gaps between the lines as high as the lines. This spacing
	        	//doesn't use the maximum of available space but half of the available
        		//space
        		int     spacing = (int)(height/(2*values.length)),
	        		unit	= (int)( (0.125)*width ),
		        	x 		= unit,
			        heightStripe	= spacing,
        			scale;


                	if ( maxValue == 0 ) {
		        	scale = 1;
        		}
	        	else {
        			scale = (width - unit) / maxValue;
		        }

        		for (int i = 0; i < values.length; ++i){
	        		int y 			= ( 2 * i ) * spacing,
		        		widthStripe = values[i] * scale;

	        		if ( maxValue < 0 ) {
		        		widthStripe = (-1) * widthStripe;
			        }
        			drawStripe(g, x, y, widthStripe, heightStripe, false);
	        	}

	        	drawArray(g, 0, 0, unit-4, spacing, false, 0, 0, values);

	        	//do the actual paint of the changed image
		        fThread.fImages.paintAnimation(image);
                }
        }
}
