package project;

/**
 *	This class is the basic class that provides the necessary methods for explaining the steps
 *	of the sorting algorithm. Every SortAlgThread has one status drawer and uses it to animate
 *	its steps.
 *
 *	@author			Hans Stenten
 *	@author			Filip Van Rysselberghe
 *	@version		1.10, 21/11/2000
 */

import java.awt.Image;
import java.awt.Graphics;
import java.awt.Color;


public class AlgStatusDrawer {
	private AlgThread fThread;
        private AlgAnimatorConfig fConfig;

	/**
	 *
	 *	Constructor
	 *
	 *	@author			Hans Stenten
	 *	@param			thread	containing sort-thread, initializes fThread
         *      @param                  configuration contains the current configuration of the application
	 */
	public AlgStatusDrawer(AlgThread thread, AlgAnimatorConfig configuration) {
		fThread = thread;
                fConfig = configuration;
                animate();
      	}

        /**
	 *	This animate function prints the strings onto the status panel
         *      explaining the current step
	 *
	 *	@author			Hans Stenten
	 *
	 *	@param			s[]     strings to be printed
	 */
        public void animate(String[] s, long pass){
                try{
                        fThread.fImages.refreshStatusPanel(s, pass);
                        this.sleep();
                }
                catch(InterruptedException e){
                }
        }

        /**
         *      This animate function sets an iniate state for the status panel
         */
        public void animate(){
                fThread.fImages.initStatusPanel();
        }

	/**
	 *
	 *	The sorting-thread will be stopped for a while so that the user can try to
	 *	figure out what's really happening
	 *
	 *	@author	Filip Van Rysselberghe
	 */
	private void sleep() throws InterruptedException{
		if (fConfig.isStep()){
			//when the thread has to evolve in steps the thread won't be put to sleep
			//but will be suspended until the user presses the step button one more
			//time
			fThread.suspend();
		}
		else{
			//in all other cases the thread will be put to sleep for a while. The time
			//that this has to last has been specified in the configuration.
			fThread.sleep(fThread.fConfig.getStepTime());
		}
	}
}