package project;

/**
 *	This class is the basic class that provides the necessary methods for animating
 *	the source code of the sorting algorithm. Every SortAlgThread has one text drawer and uses it to animate
 *	its steps.
 *
 *	@author			Hans Stenten
 *	@author			Filip Van Rysselberghe
 *	@version		1.10, 21/11/2000
 */
import java.awt.*;
import java.io.*;
import java.net.*;

public class AlgTextDrawer {
	//source text in which the active lines will be highlighted
	private Text fText;

	private AlgThread fThread;
        private AlgAnimatorConfig fConfig;
        private boolean fAnimate;


	/**
	 *
	 *	Constructor
	 *
	 *	@author			Hans Stenten
	 *	@param			thread	containing sort-thread, initializes fThread
         *      @param                  configuration contains the current configuration of the application
         *      @param                  animate animate algorithm, yes or no
	 */
        public AlgTextDrawer(AlgThread thread, AlgAnimatorConfig configuration) {
		fThread = thread;
                fConfig = configuration;
                fAnimate = !fConfig.isCompare();

                try {
                        System.err.println("init text");
                	initText(configuration);
                        System.err.println("text initiated");
                        animate();
                }
        	catch (Exception e) {
	        }
        }

        /**
	 *      Dislays the code on the textpanel
	 *
	 *	@author			Hans Stenten
	 *
	 */
	private void animate() throws InterruptedException{
                if (fAnimate){
                        System.err.println("text animate()");
                        Image 	image 	= fThread.fImages.getTextImage();
                        Graphics g = image.getGraphics();

                        // -4 to take windows borders into account
        		int width  = fConfig.width/2 - 4;
                        // -150 to take statuspanel into account
                        // -90 to take controlbar into account
	        	int height = fConfig.height - 150 - 90;
			
			g.setColor( Color.lightGray );
        		g.fillRect( 0, 0, width, height );
			
			//starting coordinates
	        	int x = 2, y = 14;
			
			//calculate max number of lines
			int maxNrLines = height / 14;
			
			
			int 	beginLine	= 0,
				endLine		= fText.length() - 1;
			
			//if the length of the text is bigger than the maximum number
			//of displayable lines. The last line printed isn't the last
			//line of the actual text. So we change lastLine;
			if ( fText.length() > maxNrLines ) {
				endLine = maxNrLines - 1;
			}
			

		        //draw the text, line after line, in the color black.
			//starting at the first and ending at the last line which
			//isn't necessary the last line of the text.
	        	g.setColor( Color.black );
        		for ( int i = beginLine; i <= endLine; i++) {
	        		g.drawString( fText.getLine(i), x, y );

        			//jump one line
	        		y += 14;
		        }

        		//paint the actual changes on the screen
	        	fThread.fImages.paintText(image);
                }
	}

        /**
	 *	This first animate function animates the drawing of the source code. It will
	 *	highlight the necessary lines. Because we want to make a difference between
	 *	decisions(like IF-structures) taken or not taken, there are two colors.
	 *	So this class demands the client to provide the active linenumber and whether
	 *	the decision was taken or not.
	 *
	 *	@author			Hans Stenten
	 *	@author 		Filip Van Rysselberghe
	 *
	 *	@param			activeLine			the linenumber of the active line
	 *	@param			decisionTaken 	wheter or not a decision is taken
	 */
	public void animate(int activeLine, boolean decisionTaken) throws InterruptedException{
                if (fAnimate){
        		Image 	image 	= fThread.fImages.getTextImage();
                        Graphics g = image.getGraphics();

	        	// -4 to take windows borders into account
        		int 		width  = fConfig.width/2 - 4;
                        // -150 to take statuspanel into account
                        // -90 to take controlbar into account
	        	int		height = fConfig.height - 150 - 90;

			g.setColor( Color.lightGray );
        		g.fillRect( 0, 0, width, height );

	        	//starting coordinates
	        	int     x = 2,
        			y = 14;

			
			//calculate maximal number of lines
			int maxNrLines = height / 14;
			
			if  ( maxNrLines > fText.length() ) {
				maxNrLines = fText.length();
			}
			
			int 	beginLine,
				endLine;
			
			if ( ( activeLine >= maxNrLines ) && ( activeLine <= (fText.length()-1) ) ) {
				beginLine = activeLine - maxNrLines;
				if ( activeLine == (fText.length()-1) ) { 
					endLine = fText.length()-1;
				}
				else {
					endLine = activeLine + 1;
				}
			}
			else {
				beginLine = 0;
				endLine = maxNrLines - 1;
			}
			
			//draw the text, line after line, in the color black.
        		//the highlighted lines will be painted in red / green.
	        	g.setFont( new Font((g.getFont()).getName(), Font.PLAIN, 12) );
			g.setColor( Color.black );
		        for ( int i = beginLine; i <= endLine; i++) {
        			if ( i == activeLine ) {
	        			if( decisionTaken ){
		        			g.setColor( Color.green );
			        	}
				        else{
        					g.setColor( Color.red );
	        			}
		        	}

	        		g.drawString( fText.getLine(i), x, y );

	        		//restore original textcolor after highlighted line
		        	g.setColor( Color.black );

	        		//jump one line
		        	y += 14;
	        	}

		        //paint the actual changes on the screen
        		fThread.fImages.paintText(image);

	        	//stop the algorithm for a while
        		this.sleep();
                }
        }

	/**
	 *	initText will open the txt-file that contains the source-code for the
	 *	algorithm that's animated.
	 *
	 *	@author		Hans Stenten
	 *	@param		configuration		the configuration from which the name
	 *													and location of the file can be
	 *													derived
	 *	@exception	IOException thrown when there is a problem opening the file
	 *				containing the Algorithm code
	 */
	private void initText(AlgAnimatorConfig configuration) throws IOException{
                if(fAnimate){
        		String 	sourceFile 	= configuration.getAlgorithmName() + ".txt";
	        	URL    	homeURL  	= configuration.getCodeBase(),
		        	sourceURL;

        		sourceURL = new URL(homeURL, sourceFile);
	        	fText = new Text(sourceURL);
        	}
        }

	/**
	 *
	 *	The sorting-thread will be stopped for a while so that the user can try to
	 *	figure out what's really happening
	 *
	 *	@author	Filip Van Rysselberghe
	 */
	private void sleep() throws InterruptedException{
		if (fConfig.isStep()){
			//when the thread has to evolve in steps the thread won't be put to sleep
			//but will be suspended until the user presses the step button one more
			//time
			fThread.suspend();
		}
		else{
			//in all other cases the thread will be put to sleep for a while. The time
			//that this has to last has been specified in the configuration.
			fThread.sleep(fThread.fConfig.getStepTime());
		}
	}

}
