package project;

import java.lang.*;
import java.io.*;
import java.util.Random;

/**
 *	Class that represents the data of the sorting algorithm
 *
 *	@author: Filip Van Rysselberghe
 */
public class Data {
	private int[] fData;

	/**
	 * 	It's quite easy to consctruct a Data-object with this
	 *	constructor. The client provides the number of integer-
	 *	values he want to be created and in wich way he wants them.
	 *	He can choose between ascending order, descending, random or
	 *	all identical.
	 *
	 *	@author	Filip Van Rysselberghe
	 *	@param	order					order of the data-values. There
	 *									are four possibilities: ascending,
	 *									descending, equal and random.
	 *	@param	int nrOfItems			number of data-values that have
	 *									to be created.

	 */
	public Data(int order, int nrOfItems ){
		this.fill(order);
	}

	/**
	 *	Fills the Data-field with values. These values can be
	 *	ordered (ascending or descending), all the same(identical)
	 *	or randomly chosen.
	 *	0	random (default)
	 *	1	ascending
	 *	2	descending
	 *	3	equal
	 *
	 *	@author	Filip Van Rysselberghe
	 *	@author	Hans Stenten
	 *	@param	int order					order
	 */
	public void fill(int order){
		switch( order ){
			case(0):
				fData = generateRandomData();
				break;
			case(1):
				fData = generateAscendingData();
				break;
			case(2):
				fData = generateDescendingData();
				break;
			case(3):
				fData = generateEqualData();
				break;
			default:
				fData = generateRandomData();
		}
	}


	/**
	 *	Generates a collection of random values
	 *
	 *	@author			Hans Stenten
	 *	@return			void
	 */
	private static int[] generateRandomData() {
		//create new array
		int[] a = new int[20];
		Random generator = new Random(System.currentTimeMillis());
		//fill it with some random numbers.
		for ( int i = 0; i < 20; i++ ) {
			a[i] = (int)Math.abs(generator.nextGaussian()*30);
		}
		return a;
	}

	/**
	 *	Generates an array of values in which the values are stored
	 *	in an ascending order.
	 *
	 * 	@author			Filip Van Rysselberghe
	 *	@return			int[]
	 */
	private static int[] generateAscendingData() {
		//create new array
		int[] a = new int[20];

		//fill array. Maxvalue 99
		for ( int i = 0; i<20; i++ ) {
			a[i] = (int)((i+1) * 4.95);
		}
		return a;
	}

	/**
	 *	generates an array of values, which is ordered in a descending
	 *	manner.
	 *
	 *	@author			Filip Van Rysselberge
	 *	@return 		int[]
	 */
	private static int[] generateDescendingData() {
		//create new array
		int[] a = new int[20];

		//fill array. Maxvalue 99
		for ( int i = 0; i<20; i++ ) {
			a[19-i] = (int)((i+1) * 4.95);
		}
		return a;
	}

	/**
	 *	generates an array filled with identical values
	 *
	 *	@author			Hans Stenten
	 *	@return 		int[]
	 */
	private static int[] generateEqualData() {
		//create new array
		int[] a = new int[20];

		//fill array.
		for ( int i = 0; i<20; i++ ) {
			a[i] = 50;
		}
		return a;
	}

	/**
	 *	What's the sense of a data class if you can't access the data
	 *	values stored inside the Data-object. Using the provided
	 *	location, the corresponding data-value stored in the obeject
	 *	will be returned. The location-system works just like with
	 *	arrays. The first item is the item on position 0. A Data-
	 *	object of length 10 thus has a maximal index of 9.
	 *
	 *	@author	Filip Van Rysselberghe
 	 *	@param	int location		the location of the data-value
 	 *
 	 *	@exception
	 */
	public int getValue( int location ){
		return fData[location];
	 }

	/**
	 *	The contrary of the getter. It sets a certain value in a
	 *	specified position.
	 *
	 *	@author	Filip Van Rysselberghe
	 *	@param	int location			location to add data
	 *	@param	int value				value to set
	 */
	public void setValue( int location, int value ){
		try {
			fData[location] = value;
		}
		//catch ( IndexOutOfBounds e ){
		//	throw e;
                catch (Exception e){
                  System.out.println("setValue error: loc("+location+") val("+value+")");
		}
	}

	/**
	 *	Returns the number of values stored in the Data-object
	 *
	 *	@author	Filip Van Rysselberghe
	 *	@return	int						number of values stored
	 */
	public int length() throws IndexOutOfBoundsException{
		return fData.length;
	}

	/**
	 *	Converts a data.object into an array of integer values.
	 *
	 *	@author	Filip Van Rysselberghe
	 *	@return	int[]					values that were stored in the
	 *									data-object in the same order
	 */
	public int[] toArray(){
		return fData;
	}

	/**
	 *	returns maximum value in the object
	 *
	 *	@author			Hans Stenten
	 *	@return			int				maximum
	 */
	public int getMaxValue(){
		int i, max=0;
		for (i=0;i<20;++i){
			if (max < fData[i] ){
				max = fData[i];
			}
		}
		return max;
	}

};