package project;

/**
 *	Thread that will perform the execution of the dining philosophers
 *	algorithm
 */
public class DiningPhilosophersAlgThread extends AlgThread {

	//necessities for the algorithm
	private Philosopher[] phils = new Philosopher[5];

	private int[] state = new int[5];
	private Semaphore mutex = new Semaphore(1);
	private Semaphore[] s = { new Semaphore(), new Semaphore(), new Semaphore(), new Semaphore(), new Semaphore()};

	/**
	 *	Constructor. Initiates fields of the object.
	 *
	 *	@param ImagesThread		images on wich can be drawn
	 *	@param AlgAnimatorConfig	configuration of current algorithm
	 */
	public DiningPhilosophersAlgThread(ImagesThread images, AlgAnimatorConfig config){
		super(images, config);
		
	}


	/**
	 *	Every class that has been derived from Thread has to contain a
	 *	run-method. This method will be called when the thread is started.
	 *	So this method will start the real Dining Philosophers algorithm
	 *
	 *	@author			Filip Van Rysselberghe
	 */
	public void run() {
		AlgDiningPhilAnimDrawer animDraw = new AlgDiningPhilAnimDrawer(this, fConfig);
		
		for (int i=0; i < 5; i++){
			phils[i] = new Philosopher(i, state, mutex, s, animDraw);
			phils[i].start();
		}
		
		//keep the thread running until the user presses the stop button
		for (;;){
		}
	}
	
	/**
	 *	Overloaded function of AlgThread. Because stopping a Dining 
	 *	Philosopher Algorithm just by stopping the mean thread isn't
	 *	sufficient because there 're philosopher threads running 
	 *	indepently who will keep running until the VM shuts down. SO 
	 *	we have to stop all the running Philosopher threads and then stop the
	 *	current DiningPhilosophersAlgThread
	 *
	 */
	public void stopAlgorithm() {
		try {
			for (int i=0; i < 5; i++){
				phils[i].stop();
			}
		}
		catch ( NullPointerException e) {
			//when a philosopher wasn't initialised the stopping 
			//will result in a nullpointer exception
		}
		
		this.stop();
	}


}
