package project;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

/**
 *	This class is the frame containing our help. This help gives
 *	user some explaination on the usage of the program. The frame
 *	consists of three buttons which are used to scroll thru the
 *	pages. There's also a ScrollPane in which the panel with the
 *	explaintion or the index will be placed.
 *	Eventhandling of the buttons or labels happens where they
 *	are declared.
 *
 *	@author Filip Van Rysselberghe
 */
public class HelpFrame extends Frame{
        private int firstLine = 0;
        private int lastLine = 20;
        private Text currentSubject;
        private String currentTitle;
        private boolean scrolling = false;

	private URL fFileLocation;
	private int fCurrentPage;
        Panel generalPanel = new Panel();
        Panel paddingBottom = new Panel();
        Panel paddingLeft = new Panel();
        Panel paddingRight = new Panel();
        BorderLayout borderLayout1 = new BorderLayout();
        Panel buttonPanel = new Panel();
        Panel bodyPanel = new Panel();
        Button indexButton = new Button();
        Button vorigeButton = new Button();
        Button volgendeButton = new Button();
        GridLayout gridLayout1 = new GridLayout();
        BorderLayout borderLayout2 = new BorderLayout();
        Scrollbar scrollbar1 = new Scrollbar();
        Panel fScroller = new Panel();
        BorderLayout borderLayout3 = new BorderLayout();
        Panel paddingPanel = new Panel();
        Label padding = new Label();
        Label sequentialSearch = new Label();
        Panel indexPanel = new Panel();
        Label insertionsort = new Label();
        Label binarySearch = new Label();
        Label quickSort = new Label();
        GridLayout gridLayout2 = new GridLayout();
        Label bubbleSort = new Label();
        Label algemeneBediening = new Label();
        Label diningPhilosophers = new Label();
        Panel paddingTop = new Panel();


        /**
         * Constructor of the frame, takes a URL pointing to the directory were
         * help files can be found. And makes the helpframe
         *
         * @param       directory       pointing to the help files directory
         *
         * @author      Filip Van Rysselberghe
         * @author      Hans Stenten
         */
        public HelpFrame( URL directory ) {
                try {
                        fFileLocation = directory;
                        jbInit();
                }
                catch(Exception e) {
                        e.printStackTrace();
                }
        }


	/**
         *	Eventhandler for the frame itself. Eventhandling for the
	 *	buttons is performed wherever they are declared. This
	 *	function only reacts on a window-destroy event by disposing
	 *	itself
         *
         * 	@author Filip Van Rysselberghe
         */
	public boolean handleEvent(Event e ) {
		if ( e.id == Event.WINDOW_DESTROY) {
			dispose();
		}
		return super.handleEvent( e );
	}

	/**
	 *	Turns to the next(previous) page. Uses the fCurrentPage in
	 *	doing so. Actually changing a page is the same as adjusting
	 *	the fCurrentPage and displaying it.
	 *
	 *	@param isGreater	wheter or not the next page has a greater
	 *			        ordernumber.
	 *	@author Filip Van Rysselberghe
	 */
	public void switchPage(boolean isGreater){
		if (isGreater) {
			++fCurrentPage;
		}
		else {
			--fCurrentPage;
		}
                scrolling = true;
		if (fCurrentPage < 0){
			fCurrentPage = 7;
		}
		else if (fCurrentPage > 7){
			fCurrentPage = 0;
                        scrolling = false;
		}

		fScroller.removeAll();
		switch (fCurrentPage) {
			case 0:
				setIndex();
				break;
			case 1:
				showSubject("Algemene Bediening", getHelpFile("AlgemeneBediening.help"));
				break;
			case 2:
				showSubject("Bubblesort", getHelpFile("Bubblesort.help"));
				break;
			case 3:
				showSubject("Insertionsort", getHelpFile("Insertionsort.help"));
				break;
			case 4:
				showSubject("Quicksort", getHelpFile("Quicksort.help"));
				break;
			case 5:
				showSubject("Sequential Search", getHelpFile("Sequentialsearch.help"));
				break;
			case 6:
				showSubject("Binary Search", getHelpFile("Binarysearch.help"));
				break;
			case 7:
				showSubject("Dining Philosophers", getHelpFile("DiningPhilosophers.help"));
				break;
			default:
		}
	}

	/**
	 *	Shows a certain subject (helpsubject) on the screen. First of all
	 *	it creates a panel and adds the title of the text. Then the
	 *	text is added below the title. Below the text comes an extra
	 *	line which you could see asa seperator.
	 *
	 *	@param	String	the title of the subject
	 *	@param	Text	the text corresponding the subject
	 *
	 *	@author Filip Van Rysselberghe
	 */
	public void showSubject(String title, Text subject){
                currentSubject = subject;
                currentTitle = title;
		int estimatedHeight = ( (subject.length()+1) * 14 ) + 80 + 45 + 20;
		final int realHeight = (estimatedHeight<300)?300:estimatedHeight;

		//maybe the solution for our scrollpane problem
		ImagePanel textField = new ImagePanel();

		textField.setLocation( 0,0 );
		textField.setLayout(null);
		textField.setSize(320, realHeight);

		textField.setBackground(Color.white);
		textField.setVisible(true);

		fScroller.removeAll();
		fScroller.add(textField);

		//drawing of the subject
		//->First we'll create an image using the panel on which
		//we 're going to draw.
		Image img = textField.createImage(textField.getSize().width, textField.getSize().height);
		Graphics g = img.getGraphics();

		//->title
		g.setFont( new Font("sansserif", Font.BOLD, 20) );
		g.setColor( Color.orange );
		g.drawString( title, 40, 40);
                printText(g, subject);

		textField.paint(img);
	}

        /**
	 *	Creates a panel with the index of our help on.
	 *	The indexwords are labels which are clickable.
	 *	The eventhandling of this labels happens within
	 *	this function.
	 *
	 *	@author Filip Van Rysselberghe
         *      @author Hans Stenten
	 */
        public void setIndex(){
                padding.setText("   ");
                sequentialSearch.setAlignment(1);
                sequentialSearch.setFont(new java.awt.Font("Dialog", 0, 16));
                sequentialSearch.setText("Sequential Search");
                sequentialSearch.addMouseListener(new java.awt.event.MouseAdapter() {
                        public void mouseClicked(MouseEvent e) {
                                sequentialSearch_mouseClicked(e);
                        }
                });
                indexPanel.setLayout(gridLayout2);
                insertionsort.setAlignment(1);
                insertionsort.setFont(new java.awt.Font("Dialog", 0, 16));
                insertionsort.setText("Insertionsort");
                insertionsort.addMouseListener(new java.awt.event.MouseAdapter() {
                        public void mouseClicked(MouseEvent e) {
                                insertionsort_mouseClicked(e);
                        }
                });
                binarySearch.setAlignment(1);
                binarySearch.setFont(new java.awt.Font("Dialog", 0, 16));
                binarySearch.setText("Binary Search");
                binarySearch.addMouseListener(new java.awt.event.MouseAdapter() {
                        public void mouseClicked(MouseEvent e) {
                                binarySearch_mouseClicked(e);
                        }
                });
                quickSort.setAlignment(1);
                quickSort.setFont(new java.awt.Font("Dialog", 0, 16));
                quickSort.setText("Quicksort");
                quickSort.addMouseListener(new java.awt.event.MouseAdapter() {
                        public void mouseClicked(MouseEvent e) {
                                quickSort_mouseClicked(e);
                        }
                });
                gridLayout2.setRows(7);
                bubbleSort.setAlignment(1);
                bubbleSort.setFont(new java.awt.Font("Dialog", 0, 16));
                bubbleSort.setText("Bubblesort");
                bubbleSort.addMouseListener(new java.awt.event.MouseAdapter() {
                        public void mouseClicked(MouseEvent e) {
                                bubbleSort_mouseClicked(e);
                        }
                });
                algemeneBediening.setAlignment(1);
                algemeneBediening.setFont(new java.awt.Font("Dialog", 0, 16));
                algemeneBediening.setText("Algemene Bediening");
                algemeneBediening.addMouseListener(new java.awt.event.MouseAdapter() {
                        public void mouseClicked(MouseEvent e) {
                                algemeneBediening_mouseClicked(e);
                        }
                });
                diningPhilosophers.setAlignment(1);
                diningPhilosophers.setFont(new java.awt.Font("Dialog", 0, 16));
                diningPhilosophers.setText("Dining Philosophers");
                diningPhilosophers.addMouseListener(new java.awt.event.MouseAdapter() {
                        public void mouseClicked(MouseEvent e) {
                                diningPhilosophers_mouseClicked(e);
                        }
                });

                fScroller.add(paddingPanel, BorderLayout.NORTH);
                paddingPanel.add(padding);
                indexPanel.add(algemeneBediening);
                indexPanel.add(bubbleSort);
                indexPanel.add(insertionsort);
                indexPanel.add(quickSort);
                indexPanel.add(binarySearch);
                indexPanel.add(sequentialSearch);
                indexPanel.add(diningPhilosophers);
                fScroller.add(indexPanel, BorderLayout.CENTER);
                pack();
                validate();
                show();
        }

        /**
         *      This adds the text of the subject below the title.
         *      The help panel has a maximum of 21 lines it can display,
         *      this function makes sure the correct lines of text are displayed.
         *      The upper and lower limit are determined by the scrollbar.
         *
         *      @param  g       graphics of the imagepanel, text is drawn on this one
         *      @param  subject the text this function displays
	 *
	 *	@author Filip Van Rysselberghe
         *      @author Hans Stenten
	 */
        private void printText(Graphics g, Text subject){
                // neem aan 300 hoog, 320 breed
                //->text
                int limit;
                if (lastLine > subject.length()){
                        limit = subject.length();
                }
                else{
                        limit = lastLine;
                }
		g.setFont( new Font("Helvetica", Font.PLAIN, 12) );
		g.setColor( Color.black);
		int	x = 20,
			y = 80,
			linefeed = 14;

		for (int i=firstLine; i < limit; i++) {
			g.drawString( subject.getLine(i), x, y );
			y += linefeed;
		}

		//draw a line below the subject's text
		g.setColor( Color.orange );
		g.fillRect( x+40, y+20, 200, 5);

        }


	/**
	 *	Tries to open a HelpFile using the fileDirectory (which contains
	 *	the base-directory of all HelpFiles) and the filename of the
	 *	Help-file
	 *
	 *	param String 			filename of helpfile
	 *	return Text			text found
	 *
	 *	@author Filip Van Rysselberghe
	 */
	private Text getHelpFile(String filename){
		try {
			URL fullFileLocation = new URL(fFileLocation, filename);

			return new Text(fullFileLocation);
		}
		catch ( MalformedURLException e) {
			return new Text();
		}
	}

	/**
         *      The size of the help frame (since it is'n resizable) 400x450
         *
	 *      @author Filip Van Rysselberghe
	 */
	public Dimension preferredSize(){
		return new Dimension(400,450);
        }

        /**
         * Initializes all components of the helpframe
         *
         * @author      Hans Stenten
         */
        private void jbInit() throws Exception {
                generalPanel.setLayout(borderLayout1);
                indexButton.setLabel("index");
                indexButton.addMouseListener(new java.awt.event.MouseAdapter() {
                        public void mouseClicked(MouseEvent e) {
                                indexButton_mouseClicked(e);
                        }
                });
                vorigeButton.setLabel("vorige");
                vorigeButton.addMouseListener(new java.awt.event.MouseAdapter() {
                        public void mouseClicked(MouseEvent e) {
                                vorigeButton_mouseClicked(e);
                        }
                });
                volgendeButton.setLabel("volgende");
                volgendeButton.addMouseListener(new java.awt.event.MouseAdapter() {
                        public void mouseClicked(MouseEvent e) {
                                volgendeButton_mouseClicked(e);
                        }
                });
                buttonPanel.setLayout(gridLayout1);
                bodyPanel.setLayout(borderLayout2);
                scrollbar1.addAdjustmentListener(new java.awt.event.AdjustmentListener() {
                        public void adjustmentValueChanged(AdjustmentEvent e) {
                                scrollbar1_adjustmentValueChanged(e);
                        }
                });
                this.setTitle("Hulp bij Algorithms in Action");
                this.setResizable(false);
                scrollbar1.setMaximum(20);
                scrollbar1.setVisibleAmount(1);
                fScroller.setLayout(borderLayout3);
                paddingTop.setBackground(Color.lightGray);
                paddingRight.setBackground(Color.lightGray);
                paddingLeft.setBackground(Color.lightGray);
                paddingBottom.setBackground(Color.lightGray);
                buttonPanel.setBackground(Color.lightGray);
                this.add(generalPanel, BorderLayout.CENTER);
                generalPanel.add(buttonPanel, BorderLayout.NORTH);
                buttonPanel.add(indexButton);
                buttonPanel.add(vorigeButton);
                buttonPanel.add(volgendeButton);
                generalPanel.add(bodyPanel, BorderLayout.CENTER);
                bodyPanel.add(fScroller, BorderLayout.CENTER);
                bodyPanel.add(scrollbar1, BorderLayout.EAST);
                this.add(paddingBottom, BorderLayout.SOUTH);
                this.add(paddingLeft, BorderLayout.WEST);
                this.add(paddingRight, BorderLayout.EAST);
                this.add(paddingTop, BorderLayout.NORTH);
                setIndex();
        }

        /**
         * Action handling function
         * Turns to the next page.
         *
         * @author      Hans Stenten
         */
        void volgendeButton_mouseClicked(MouseEvent e) {
                switchPage(true);
        }

        /**
         * Action handling function
         * Turns to the previous page
         *
         * @author      Hans Stenten
         */
        void vorigeButton_mouseClicked(MouseEvent e) {
                switchPage(false);
        }

        /**
         * Action handling function
         * Returns to the index page
         *
         * @author      Hans Stenten
         */
        void indexButton_mouseClicked(MouseEvent e) {
                fScroller.removeAll();
                setIndex();
        }

        /**
         * Action handling function
         * Scrolls through the text, this function adjusts the limit lines needed in printText
         *
         * @author      Hans Stenten
         */
        void scrollbar1_adjustmentValueChanged(AdjustmentEvent e) {
                if (scrolling){
                        firstLine = e.getValue();
                        lastLine = firstLine + 20;
                        showSubject(currentTitle, currentSubject);
                }
        }

        /**
         * Action handling function
         * Turns to the Algemene Bediening page
         *
         * @author      Hans Stenten
         */
        void algemeneBediening_mouseClicked(MouseEvent e) {
                fScroller.removeAll();
                showSubject("Algemene Bediening", getHelpFile("AlgemeneBediening.help"));
                scrolling = true;
        }

        /**
         * Action handling function
         * Turns to the bubblesort page
         *
         * @author      Hans Stenten
         */
        void bubbleSort_mouseClicked(MouseEvent e) {
                fScroller.removeAll();
                showSubject("Bubblesort", getHelpFile("Bubblesort.help"));
                scrolling = true;
        }

        /**
         * Action handling function
         * Turns to the insertionsort page
         *
         * @author      Hans Stenten
         */
        void insertionsort_mouseClicked(MouseEvent e) {
                fScroller.removeAll();
                showSubject("Insertionsort", getHelpFile("Insertionsort.help"));
                scrolling = true;
        }

        /**
         * Action handling function
         * Turns to the quicksort page
         *
         * @author      Hans Stenten
         */
        void quickSort_mouseClicked(MouseEvent e) {
                fScroller.removeAll();
                showSubject("Quicksort", getHelpFile("Quicksort.help"));
                scrolling = true;
        }

        /**
         * Action handling function
         * Turns to the binary search page
         *
         * @author      Hans Stenten
         */
        void binarySearch_mouseClicked(MouseEvent e) {
                fScroller.removeAll();
                showSubject("Binary Search", getHelpFile("Binarysearch.help"));
                scrolling = true;
        }

        /**
         * Action handling function
         * Turns to the sequential search page
         *
         * @author      Hans Stenten
         */
        void sequentialSearch_mouseClicked(MouseEvent e) {
                fScroller.removeAll();
                showSubject("Sequential Search", getHelpFile("Sequentialsearch.help"));
                scrolling = true;
        }

        /**
         * Action handling function
         * Turns to the dining philosophers page
         *
         * @author      Hans Stenten
         */
        void diningPhilosophers_mouseClicked(MouseEvent e) {
                fScroller.removeAll();
                showSubject("Dining Philosophers", getHelpFile("DiningPhilosophers.help"));
                scrolling = true;
        }
}

