package project;

import java.awt.*;
import java.io.*;
import java.util.*;

/**
 * 	ImagePanel has all the characteristics of a normal panel but also
 *	contains an image that can be used in painting the contents of
 *	the panel in a more performant way. Paint(Image i) will repaint
 *	the whole panel with the new image i. By calling GetImage(), the
 *	existing Image can be obtained and altered.
 *
 *	@author Filip Van Rysselberghe
 *	@version 1.0 28/11/2000
 *
 *	@bugs none known
 */


public class ImagePanel extends Panel{
	private Image fImage = null;

	/**
	 *	This is the constructor of the panel. The background will be
	 *	set to lightGray by default.
	 *
	 *	@author Filip Van Rysselberghe
	 */
	public ImagePanel() {
		setBackground( Color.lightGray );
		validate();
	}

	/**
	 *	The paint function paints the image(if it exists) on the panel
	 *
	 *	@author 	Filip Van Rysselberghe
	 *	@param 		Graphics g				Graphics-object painted on
	 */
	public void paint(Graphics g){
		if ( fImage != null ){
			g.drawImage(fImage,0,0,null);
		}
	}

	/**
	 *	Stores a new image in the object and will ask to repaint the
	 *	panel using the new image.
	 *
	 *	@author 	Filip Van Rysselberghe
	 *	@param		Image i					new Image to be painted
	 */
	public void paint(Image i){
		fImage = i;
		repaint();
	}

	/**
	 *	Used for accesing the image stored in the panel. Since it's
	 *	better for an efficient way of drawing to use the new image
	 *	instead of creating a new image all the time.
	 *
	 *	@author		Filip Van Rysselberghe
	 *	@return		Image					Image stored in the panel
	 */
	public Image getImage(){
		return fImage;
	}
}
