package project;

/**
 * This thread contains all panels used in the applet, and has functions to update those
 * panels. We use this construction because letting a thread (the algorithm thread) alter
 * or update the panels, is asking for troubles. So this thread contains those panels.
 *
 * @see AlgThread
 * @author Hans Stenten
 */

import java.awt.Dimension;
import java.awt.Image;
import java.awt.Panel;

public class ImagesThread extends Thread {
  private Panel[] fPanels;
  private AlgAnimatorConfig fConfig;

  /**
   * Constructor for the image thread
   *
   * @param panels      the different panels used in the applet
   * @param config      the configuration of the applet, needed to update the panels
   *
   * @author Hans Stenten
   */
  public ImagesThread(Panel[] panels, AlgAnimatorConfig config) {
          fPanels = panels;
          fConfig = config;
  }

  /**
   * Resets the panels
   *
   * @param panels      the different panels used in the applet
   * @param config      the configuration of the applet, needed to update the panels
   *
   * @author Hans Stenten
   */
  public void setComponents(Panel[] panels, AlgAnimatorConfig config){
          fPanels = panels;
          fConfig = config;
  }

  /**
   * Returns the image of the animation panel
   *
   * @author Hans Stenten
   * @author Filip Van Rysselberghe
   */
  public Image getAnimationImage(){
      int	height = (fPanels[0].getSize()).height,
      		width = (fPanels[0].getSize()).width;

      //check wether an image exists
      if (((ImagePanel)fPanels[0]).getImage() == null){
      	//return fPanels[0].createImage(fConfig.width/2 -4, fConfig.height-80);
      	return fPanels[0].createImage(width, height);
      }

	  //get image. Before returning we'll check the size of the Image against
	  //the size of our Panel
	  Image i = ((ImagePanel)fPanels[0]).getImage();

	  //when the entire frame was resized then we'll have to create a new
	  //image using the new size. But first check wether the size really changed
	  if ( (i.getHeight(fPanels[0]) != height ) ||
	  	(i.getWidth(fPanels[0]) != width) ) {

		//VERANDEREN!!!!!IN NEN CONVERT OFZO
		i.flush();


		i = fPanels[0].createImage(width, height);
	  }


	  return i;
	  //return ((ImagePanel)fPanels[0]).getImage();
  }

  /**
   * Returns the images of the source panel
   *
   * @author Hans Stenten
   * @author Filip Van Rysselberghe
   */
  public Image getTextImage(){
      int	height = (fPanels[0].getSize()).height,
      		width = (fPanels[0].getSize()).width;

      if (((ImagePanel)fPanels[1]).getImage() == null){
            //return fPanels[1].createImage(fConfig.width/2 -4, fConfig.height-90-150);
      		return fPanels[1].createImage(width, height);
      }

	  //get image. Before returning we'll check the size of the Image against
	  //the size of our Panel
	  Image i = ((ImagePanel)fPanels[1]).getImage();

	  //when the entire frame was resized then we'll have to create a new
	  //image using the new size. But first check wether the size really changed
	  if ( (i.getHeight(fPanels[1]) != height ) ||
	  	(i.getWidth(fPanels[1]) != width ) ) {

		//VERANDEREN!!!!!IN NEN CONVERT OFZO
		i.flush();

		//i = fPanels[1].createImage(fConfig.width/2 -4, fConfig.height-90-150);
		i = fPanels[1].createImage(width, height);
	}


	  return i;
	  //return ((ImagePanel)fPanels[1]).getImage();
  }

  /**
   * Returns the dimension of the animation panel
   *
   * @author Hans Stenten
   */
  public Dimension getAnimationDimension(){
          return new Dimension( fPanels[0].getSize() );
          //return new Dimension(fConfig.width/2 -4, fConfig.height-90);
  }

  /**
   * Returns the dimension of the source panel
   *
   * @author Hans Stenten
   */
  public Dimension getTextDimension(){
          return new Dimension( fPanels[1].getSize() );
          //return new Dimension(fConfig.width/2 -4, fConfig.height-90-150);
  }

  /**
   * Paints the animation panel
   *
   * @param i the image (see getAnimationImage) to be shown
   * @author Hans Stenten
   */
  public void paintAnimation(Image i){
          ((ImagePanel)fPanels[0]).paint(i);
  }

  /**
   * Paints the text panel
   *
   * @param i the image (see getTextImage) to be shown
   * @author Hans Stenten
   */
  public void paintText(Image i){
          ((ImagePanel)fPanels[1]).paint(i);
  }

  /**
   * Update the status panel
   *
   * @param s a list of messages to be shown
   * @param pass the current # of passes of the algorithm
   * @author Hans Stenten
   */
  public void refreshStatusPanel(String[] s, long pass){
        ((StatusPanel)fPanels[2]).statusPass.setVisible(true);
        ((StatusPanel)fPanels[2]).statusTitle.setText(fConfig.getAlgorithmName());
        ((StatusPanel)fPanels[2]).statusPass.setText("Passes : "+pass);
        if ( !fConfig.isSearch() ) ((StatusPanel)fPanels[2]).statusSearch.setVisible(false);
        if ( fConfig.isSearch() ){
                ((StatusPanel)fPanels[2]).statusSearch.setVisible(true);
                ((StatusPanel)fPanels[2]).statusSearch.setText("Searchvalue : "+fConfig.getSearchValue());
        }
        int i;
        for (i=0; i < s.length; ++i){
                ((StatusPanel)fPanels[2]).statusArea.append(s[i]+"\n");
        }
        ((StatusPanel)fPanels[2]).statusArea.append("\n");
  }

  /**
   * Initiate the status panel
   *
   * @author Hans Stenten
   */
  public void initStatusPanel(){
        ((StatusPanel)fPanels[2]).statusPass.setVisible(false);
        ((StatusPanel)fPanels[2]).statusSearch.setVisible(false);
        ((StatusPanel)fPanels[2]).statusTitle.setText(fConfig.getAlgorithmName());
        ((StatusPanel)fPanels[2]).statusArea.setText(fConfig.getAlgorithmName()+" : \n");
  }

  public void run(){
          for(;;){
          }
  }
}
