package project;

import java.awt.*;
import java.awt.event.*;
import java.net.*;


/**
 *	This class is a popup used to get a value from the user
 *      Used in searchthread, to get the searchvalue from user.
 *
 *	@author Hans Stenten
 */
public class Popup extends Frame {
        SearchAlgThread fThread;

        Panel generalPanel = new Panel();
        GridLayout gridLayout2 = new GridLayout();
        GridLayout gridLayout1 = new GridLayout();
        Label popupTitle = new Label();
        TextField popupValue = new TextField();
        Panel panel1 = new Panel();
        Button popupButton = new Button();
        BorderLayout borderLayout1 = new BorderLayout();

        /**
         * Constructor of the frame, takes a URL pointing to the directory were
         * help files can be found. And makes the helpframe
         *
         * @param       directory       pointing to the help files directory
         *
         * @author      Filip Van Rysselberghe
         * @author      Hans Stenten
         */
        public Popup(SearchAlgThread thread) {
                try {
                        fThread = thread;
                        jbInit();
                }
                catch(Exception e) {
                        e.printStackTrace();
                }
        }


	/**
         *	Eventhandler for the frame itself. Eventhandling for the
	 *	buttons is performed wherever they are declared. This
	 *	function only reacts on a window-destroy event by disposing
	 *	itself
         *
         * 	@author Filip Van Rysselberghe
         */
	public boolean handleEvent(Event e ) {
		if ( e.id == Event.WINDOW_DESTROY) {
			dispose();
		}
		return super.handleEvent( e );
	}


	/**
         *      The size of the help frame (since it is'n resizable) 400x450
         *
	 *      @author Filip Van Rysselberghe
	 */
	public Dimension preferredSize(){
		return new Dimension(250,100);
        }

        /**
         * Initializes all components of the helpframe
         *
         * @author      Hans Stenten
         */
        private void jbInit() throws Exception {
                generalPanel.setLayout(gridLayout1);
                this.setTitle("About - Algorithms in Action");
                this.setResizable(false);
                gridLayout1.setRows(3);
                popupTitle.setBackground(Color.lightGray);
                popupTitle.setText("Geef een waarde om naar te zoeken op");
                popupValue.setText("0");
                popupButton.setLabel("ok");
                popupButton.addMouseListener(new java.awt.event.MouseAdapter() {
                        public void mouseClicked(MouseEvent e) {
                                popupButton_mouseClicked(e);
                        }
                });
                panel1.setLayout(borderLayout1);
                panel1.setBackground(Color.lightGray);
                this.add(generalPanel, BorderLayout.CENTER);
                generalPanel.add(popupTitle, null);
                generalPanel.add(popupValue, null);
                generalPanel.add(panel1, null);
                panel1.add(popupButton, BorderLayout.EAST);
                pack();
                validate();
                show();
        }

        void popupButton_mouseClicked(MouseEvent e) {
                try{
                        int v = Integer.parseInt(popupValue.getText());
                        if (v >= 0){
                                fThread.setValue(v);
                                fThread.resume();
                                dispose();
                        }
                        else{
                                popupTitle.setText("Gelieve een positief getal in te geven");
                        }
                }
                catch(NumberFormatException nfe){
                        popupTitle.setText("Gelieve een getal in te geven");
                }
        }
}