package project;

/**
 *
 *	QuicksortAlgThread extends SortAlgThread
 *	with an quicksort implementation.
 *
 *	@author			Hans Stenten
 *	@version		1.10, 04/11/2000
 */

import java.awt.*;
import java.io.*;
import java.lang.*;

public class QuicksortAlgThread extends SortAlgThread{
	/**
	 *	Constructor
	 *	just calls constructor of SortAlgThread
	 *
	 *	@author			Hans Stenten
	 *	@param			images  provides the images to draw on (animation, text, status)
         *      @param                  config  the configuration of the application
         *      @see                    ImagesThread
         *      @see                    AlgAnimatorConfig
	 */
	public QuicksortAlgThread( ImagesThread images, AlgAnimatorConfig config ){
		super(images, config);
	}

	/**
	 *	java version of the quicksort algorithm as can be found in the course "Fundamenten van de Informatica" by
	 *	Prof. E Laenens.
	 *	quickSort is the actual sort function, it is a quick sort implementation.
	 *	To animate the algorithm the animate function of the drawer class is called
	 *
	 *	@author			Hans Stenten
	 *	@param			int	first	the upperbound of the array-part to be sorted this run
	 *	@param			int	last	the lowerbound of the array-part to be sorted this run
	 */
	private void quickSort(int first, int last) throws InterruptedException{
                fTextDraw.animate(0, false);    // PROCEDURE quicksort(VAR itemsToSort: ARRAY OF INTEGER; first, last: INTEGER);
                fTextDraw.animate(1, false);    // BEGIN
                fTextDraw.animate(2, false);    // VAR middleItem, i, j: INTEGER;
                fTextDraw.animate(3, false);    // VAR swap : BOOLEAN;
		int i, j, middleItem, middle;
		i = first;
		j = last;
		// determine the pivot element
                middle = (int)(first+last)/2;
		middleItem = fItemsToSort.getValue( middle );
                fTextDraw.animate(4, false);    // middle := (first+last) DIV 2;
                fTextDraw.animate(5, false);    // middleItem := itemsToSort[middle];
                fTextDraw.animate(6, false);    // i := first;
                fTextDraw.animate(7, false);    // j := last;
                fStatusDraw.animate(new String[]{"middleItem : "+middleItem, "i : "+i, "j : "+j, "middle : "+middle}, fPass);
		// put all elements smaller than element on pivot element left of pivot element
		// all elements greater right of pivot element
		do{
			fTextDraw.animate(8, true);     // REPEAT
                        fTextDraw.animate(16, true);    // UNTIL (i <= j);
			// search the first element greater than the middleItem
                        fTextDraw.animate(9, false);    // WHILE ( (itemsToSort[i] < middleItem) AND (i < LEN(itemsToSort)) ) DO INC(i) END;
			while (fItemsToSort.getValue(i) < middleItem && i < fItemsToSort.length()){
                                fTextDraw.animate(9, true);     // WHILE ( (itemsToSort[i] < middleItem) AND (i < LEN(itemsToSort)) ) DO INC(i) END;
                                fStatusDraw.animate(new String[]{"itemsToSort["+i+"] : "+fItemsToSort.getValue(i), "middleItem : "+middleItem, "LEN(itemsToSort) : "+fItemsToSort.length()}, fPass);
                                ++i;
                        }
			// search the first element smaller than the middleItem
                        fTextDraw.animate(10, false);    // WHILE ( (itemsToSort[j] > middleItem) AND (j >= 0) ) DO DEC(j) END;
			while (fItemsToSort.getValue(j) > middleItem && j > 0){
                                fTextDraw.animate(10, true);     // WHILE ( (itemsToSort[j] > middleItem) AND (j >= 0) ) DO DEC(j) END;
                                fStatusDraw.animate(new String[]{"itemsToSort["+i+"] : "+fItemsToSort.getValue(i), "middleItem : "+middleItem, "LEN(itemsToSort) : "+fItemsToSort.length()}, fPass);
                                --j;
                        }
			// swap
                        fStatusDraw.animate(new String[]{"i : "+i+"\tfItemsToSort["+i+"] : "+fItemsToSort.getValue(i), "j : "+j+"\tfItemsToSort["+j+"] : "+fItemsToSort.getValue(j), "middle : "+middle+"\tfItemsToSort["+middle+"] : "+fItemsToSort.getValue(middle)}, fPass);
			// swap the greater element left of the middleItem
			// with the smaller element right of the middleItem
                        fTextDraw.animate(11, false);   // IF (i <= j) DO
			if (i <= j){
				fTextDraw.animate(11, true);    // IF (i <= j) DO
                                fAnimDraw.animate(i, j, middle, true);
				fTextDraw.animate(12, false);   // swap(i,j);
                                fTextDraw.animate(13, false);   // INC(i);
                                fTextDraw.animate(14, false);   // DEC(j);
				swap(i,j);
                                fAnimDraw.animate(i, j, middle, true);
                                ++fPass;
				++i;
				--j;
                                fStatusDraw.animate(new String[]{"i : "+i+"\tfItemsToSort["+i+"] : "+fItemsToSort.getValue(i), "j : "+j+"\tfItemsToSort["+j+"] : "+fItemsToSort.getValue(j), "middle : "+middle+"\tfItemsToSort["+middle+"] : "+fItemsToSort.getValue(middle)}, fPass);
                                fTextDraw.animate(15, false);   // END;
			}
			//fAnimDraw.animate(i, j, false);
                        fTextDraw.animate(16, false);   // END;
		}while(i<=j);
		// if there are still elements between first and pivot element unsorted -> sort that region
                fStatusDraw.animate(new String[]{"first : "+first, "j : "+j}, fPass);
                fTextDraw.animate(17, false);
		if (first < j){
			fTextDraw.animate(17, true);
			quickSort(first, j);
		}
		// if there are still elements between pivot element and last unsorted -> sort that region
                fStatusDraw.animate(new String[]{"i : "+i, "last : "+last}, fPass);
                fTextDraw.animate(18, false);
		if (i < last){
			fTextDraw.animate(18, true);
			quickSort(i, last);
		}
	}

	/**
	 *	calls quickSort()
	 *
	 *	@author			Hans Stenten
	 */
	public void sort() throws Exception {
                fStatusDraw.animate(new String[]{"middelItem in het groen", "i en j in het rood "}, fPass);
		quickSort(0, fItemsToSort.length() - 1);
                fStatusDraw.animate(new String[]{"Items sorted in "+fPass+""}, fPass);
                fConfig.setRunning(false);
	}
}
