package project;

/**
 *
 *	SearchAlgThread is a class containing all elements needed for the
 *	implementation of a search algorithm.
 *	The search()-funtion is an abstract function and must be filled in by
 *	derived classes.
 *	Further is SearchAlgThread an extension of AlgThread which is an
 *	extension of Thread, so is runnable as a Thread.
 *
 *	@author			Hans Stenten
 *	@author			Filip Van Rysselberghe
 *	@version		1.10, 04/11/2000
 */
import java.awt.*;
import java.io.*;
import java.lang.*;
import java.util.Random;

public abstract class SearchAlgThread extends AlgThread {

        protected Data fItemsToSearch;

	protected AlgTextDrawer	fTextDraw;
        protected AlgSearchAnimDrawer fAnimDraw;
        protected AlgStatusDrawer fStatusDraw;

	protected long fPass;


        /**
	 *	Constructor, instantiates it's super class with the necessary
	 * 	information.
	 *
	 *	@author			Filip Van Rysselberghe
	 *	@param			AlgAnimatorFrame	parent	the frame containing the Thread
	 */
        protected SearchAlgThread(ImagesThread images, AlgAnimatorConfig config) {
                super(images, config);
                // std ascending
		fItemsToSearch = new Data(1, 20);
		fTextDraw = new AlgTextDrawer(this, fConfig);
                fAnimDraw = new AlgSearchAnimDrawer(this, fConfig);
                fStatusDraw = new AlgStatusDrawer(this, fConfig);
	}

	/**
	 *	Every class that has been derived from Thread has to contain a
	 *	run-method. This method will be called when the thread is started.
	 *	Since we want our thread to search the data, the run will start the
	 *	search.
	 *
	 *	@author			Filip Van Rysselberghe
	 *	@author			Hans Stenten
	 */
	public void run() {
		try {
                        Popup p = new Popup(this);
                        suspend();
			search(fConfig.getSearchValue());
		}
		catch ( Exception e ) {
		}
	}

	/**
	 *	This is the actual action that will be filled in by all
	 *	search threads. All those threads have their own special
	 *	way of searching the data. So you can call it an abstract
	 *	method (which it is!).
	 */
	public abstract void search(int value) throws Exception;

	/**
	 *	returns the passes of the algorithm
	 *
	 *	@author			Hans Stenten
	 *	@return			long
	 */
	public long getPass(){
		return fPass;
	}

	/**
	 *	sets the value to search for
	 *
	 *	@author			Hans Stenten
	 */
        public void setValue(int v){
                fConfig.setSearchValue(v);
        }
}