package project;

/**
 *
 *	SortAlgThread is a class containing all elements needed for the
 *	implementation of a sort algorithm.
 *	The sort()-funtion is an abstract function and must be filled in by
 *	derived classes.
 *	Further is SortAlgThread an extension of AlgThread which is an
 *	extension of Thread, so is runnable as a Thread.
 *
 *	@author			Hans Stenten
 *	@author			Filip Van Rysselberghe
 *	@version		1.10, 04/11/2000
 */
import java.awt.*;
import java.io.*;
import java.lang.*;
import java.util.Random;

public abstract class SortAlgThread extends AlgThread{

	protected Data fItemsToSort;

	protected AlgTextDrawer	fTextDraw;
   	protected AlgSortAnimDrawer fAnimDraw;
   	protected AlgStatusDrawer fStatusDraw;

	protected long fPass;

	/**
	 *	Constructor, instantiates it's super class with the necessary
	 * 	information.
	 *
	 *	@author			Filip Van Rysselberghe
	 *	@param			AlgAnimatorFrame	parent	the frame containing the Thread
	 */
	protected SortAlgThread(ImagesThread images, AlgAnimatorConfig config) {
		super(images, config);
                System.err.println("Sort in");
                System.err.println("items to sort");
		fItemsToSort = new Data(fConfig.getItemStyle(), 20);
                System.err.println("textdrawer");
		fTextDraw = new AlgTextDrawer(this, fConfig);
                System.err.println("animater");
                fAnimDraw = new AlgSortAnimDrawer(this, fConfig);
                System.err.println("status");
                fStatusDraw = new AlgStatusDrawer(this, fConfig);
                System.err.println("Sort out");
	}

	/**
	 *	Every class that has been derived from Thread has to contain a
	 *	run-method. This method will be called when the thread is started.
	 *	Since we want our thread to sort the data, the run will start the
	 *	sorting.
	 *
	 *	@author			Filip Van Rysselberghe
	 *	@author			Hans Stenten
	 */
	public void run() {
		try {
			sort();
		}
		catch ( Exception e ) {
		}
	}

	/**
	 *	This is the actual action that will be filled in by all
	 *	sorting threads. All those threads have their own special
	 *	way of sorting the data. So you can call it an abstract
	 *	method (which it is!).
	 */
	public abstract void sort() throws Exception;

	/**
	 *	returns maximum value in fItemsToSort
	 *
	 *	@author			Hans Stenten
	 *	@return			int		maximum
	 */
	public int getMaxValue(){
		int i, max=0;
		for (i=0;i<20;++i){
			if (max < fItemsToSort.getValue(i) ){
				max = fItemsToSort.getValue(i);
			}
		}
		return max;
	}

	/**
	 *	swaps the element on the two given positions in the array
	 *
	 *	@author			Hans Stenten
	 *	@param			int first		first position
	 *	@param			int second		second position
	 */
	public void swap(int first, int second){
		int temp = fItemsToSort.getValue(second);
		fItemsToSort.setValue(second , fItemsToSort.getValue(first));
		fItemsToSort.setValue(first, temp);
	}

	/**
	 *	returns the passes of the algorithm
	 *
	 *	@author			Hans Stenten
	 *	@return			long
	 */
	public long getPass(){
		return fPass;
	}
}
