package project;

import java.io.*;
import java.util.*;
import java.net.*;
import java.lang.*;

/**
 *	This class represents a text. Like every text that was written
 *	on a paper it exists of different lines. A text can be made by
 *	just taking a white paper(default constructor) and adding line
 *	after line. Another way is providing the URL location where the
 *	text is located.
 *
 *	@author	Filip Van Rysselberghe
 *	@verion 1.1,	08/10/2000
 *
 *	@bugs none known
 */
public class Text {
	private Vector fLine;

	/**
	 *	function that will read the DataInputStream and put
	 *	the text into the vector of lines.
	 *
	 *	@author	Filip Van Rysselberghe
	 *	@param	DataInputStream textStream	stream from which
	 *										the text will be read
	 */
	private void readInput( DataInputStream textStream ){
		String lineRead;

		try {
			while ( (lineRead = textStream.readLine()) != null ) {
				fLine.addElement( expandTabs(lineRead) );
			}
		}
		catch ( IOException e ) {
		}

	}

	/**
	 *	Since tabs will not be printed by drawString of AWT we 'll
	 *	have to expand tabs into a serie of spaces
	 *
	 *	@author	Filip Van Rysselberghe
	 *	@param	String str				string in which we will replace the tabs
	 *	@return	String					string with expanded tabs
	 */
	private String expandTabs( String str ){
		int pos;
		while ( ( pos = str.indexOf('\t') ) != -1 ) {
			StringBuffer b = new StringBuffer(str);
			b.setCharAt(pos, ' ');
			b.insert(pos, "    ");
			//System.out.println("removed tab at" + pos);
			str = b.toString();
		}
		return str;

	}
	/**
	 *	This is the default constructor. It creates an empty text.
	 *	It's like taking a white paper on which you will write your
	 *	text line by line.
	 *
	 *	@author	Filip Van Rysselberghe
	 */
	public Text(){
		fLine = new Vector();
	}

	/**
	 *	The other constructor that takes the URL location of the
	 *	text to open.
	 *
	 *	@author	Filip Van Rysselberghe
	 *	@param	URL Location			Location of the text created
	 */
	public Text( URL Location ){
		fLine = new Vector();

		InputStream input = null;
		System.out.println(Location.getFile());
		try {
			input = Location.openStream();
			System.out.println("location opened");
			
			if ( input != null ) {
				DataInputStream textStream = new DataInputStream(input);
				readInput( textStream );
				System.out.println("input read; length = "+fLine.size());
			}
		}
		catch ( IOException e ){
			System.err.println("couldn't open file");
			fLine.addElement("File Not Found");
		}

		/*if ( input != null ) {
			DataInputStream textStream = new DataInputStream(input);
			readInput( textStream );
			System.out.println("input read; length = "+fLine.size());
		}*/
	}

	/**
	 *	As the length-function in the class vector, this function
	 *	will return the length of the text. With length of a text
	 *	we mean the number of lines.
	 *
	 *	@author	Filip Van Rysselberghe
	 *	@return	int						number of lines
	 */
	public int length(){
		return fLine.size();
	}

	/**
	 *	Returns the line of the text specified by the linenumber
	 *
	 *	@author	Filip Van Rysselberghe
	 *	@param	int lineNr				number of the line we want to view
	 *	@return String					the line of text
	 */
	public String getLine( int lineNr ){
		return (fLine.elementAt( lineNr )).toString();
	}

	/**
	 *	Add an extra line after the last line of the text
	 *
	 *	@author	Filip Van Rysselberghe
	 *	@param	String line				line of text we want to add
	 */
	public void addLine( String line ) {
		fLine.addElement( line );
	}
}
